# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Setup for the tink-py package with pip.

The behavior of this script can be modified using two enviroment variables:

  - TINK_PYTHON_SETUPTOOLS_OVERRIDE_VERSION: To change the version that is used.
  - TINK_PYTHON_BAZEL_REMOTE_CACHE_GCS_BUCKET_URL: If set, Bazel Remote Caching
  is used to build the package.
  - TINK_PYTHON_BAZEL_REMOTE_CACHE_SERVICE_KEY_PATH: Service key to use for
  Bazel Remote Caching.
"""

import glob
import os
import platform
import posixpath
import re
import shutil
import subprocess
import sys
from typing import Any, List, Mapping

import setuptools
from setuptools.command import build_ext

_PROJECT_BASE_DIR = os.path.dirname(os.path.abspath(__file__))
_TINK_CRYPTO_GITHUB_ORG_URL = 'https://github.com/tink-crypto'


def _get_tink_version() -> str:
  """Returns the project version.

  If the TINK_PYTHON_SETUPTOOLS_OVERRIDE_VERSION environment variable is set,
  this function returns its value; otherwise, it parses the value in
  TINK_VERSION.txt.
  """
  if 'TINK_PYTHON_SETUPTOOLS_OVERRIDE_VERSION' in os.environ:
    return os.environ['TINK_PYTHON_SETUPTOOLS_OVERRIDE_VERSION']
  with open(os.path.join(_PROJECT_BASE_DIR, 'TINK_VERSION.txt')) as f:
    version = f.read().strip()
    if not re.fullmatch(r'[0-9]+.[0-9]+.[0-9]+', version):
      raise ValueError(f'Invalid version: {version}')
    return version


def _get_bazel_command() -> str:
  """Finds the bazel command."""
  if shutil.which('bazelisk'):
    return 'bazelisk'
  elif shutil.which('bazel'):
    return 'bazel'
  raise FileNotFoundError(
      'Could not find bazel executable. Please install '
      'bazel to compile the Tink Python package.'
  )


def _get_protoc_command() -> str:
  """Finds the protoc command."""
  if 'PROTOC' in os.environ and os.path.exists(os.environ['PROTOC']):
    return os.environ['PROTOC']
  protoc_path = shutil.which('protoc')
  if protoc_path is None:
    raise FileNotFoundError(
        'Could not find protoc executable. Please install '
        'protoc to compile the Tink Python package.'
    )
  return protoc_path


def _generate_proto(protoc: str, source: str) -> None:
  """Invokes protoc to generate a _pb2.py from .proto files."""
  if not os.path.exists(source):
    raise FileNotFoundError(f'Cannot find required file: {source}')

  output = source.replace('.proto', '_pb2.py')

  if os.path.exists(output) and os.path.getmtime(source) < os.path.getmtime(
      output
  ):
    # No need to regenerate if output is newer than source.
    return

  print(f'Generating {output}...')
  protoc_args = [protoc, '-I.', '--python_out=.', source]
  subprocess.run(args=protoc_args, check=True)


def _parse_requirements(requirements_filename: str) -> List[str]:
  """Parses requirements from the given file."""
  with open(os.path.join(_PROJECT_BASE_DIR, requirements_filename)) as f:
    return [
        line.rstrip()
        for line in f
        if not (line.isspace() or line.startswith('#'))
    ]


class BazelExtension(setuptools.Extension):
  """A C/C++ extension that is defined as a Bazel BUILD target."""

  def __init__(self, bazel_target: str, target_name: str = '') -> None:
    self.bazel_target = bazel_target
    self.relpath, self.target_name = posixpath.relpath(
        bazel_target, '//'
    ).split(':')
    if target_name:
      self.target_name = target_name
    ext_name = os.path.join(
        self.relpath.replace(posixpath.sep, os.path.sep), self.target_name
    )
    setuptools.Extension.__init__(self, ext_name, sources=[])


class BuildBazelExtension(build_ext.build_ext):
  """A command that runs Bazel to build a C/C++ extension."""

  def __init__(self, dist: str) -> None:
    super().__init__(dist)
    self.bazel_command = _get_bazel_command()

  def run(self) -> None:
    for ext in self.extensions:
      self.bazel_build(ext)
    build_ext.build_ext.run(self)

  def bazel_build(self, ext: BazelExtension) -> None:
    if not os.path.exists(self.build_temp):
      os.makedirs(self.build_temp)

    # Ensure no artifacts from previous builds are reused (i.e. from builds
    # using a different Python version).
    self.spawn([self.bazel_command, 'clean', '--expunge'])

    bazel_startup_args = []
    bazel_build_args = [
        'build',
        ext.bazel_target,
        '--compilation_mode=' + ('dbg' if self.debug else 'opt'),
    ]

    # Allow the user to select a configuration (using SETUP_PY_BAZEL_CONFIG).
    # The configuration then selects the corresponding lines in .bazelrc.
    # Used for macOS which needs different flags depending on whether we cross
    # compile
    bazel_config_arg = os.getenv('SETUP_PY_BAZEL_CONFIG', '')
    if bazel_config_arg:
      bazel_build_args += [f'--config={bazel_config_arg}']

    # Sets e.g.
    # --@rules_python//python/config_settings:python_version=3.12
    # which is the way to tell bazel which Python version to use (without this,
    # we fall back to the default as specified in MODULE.bazel)
    python_version_rule = '@rules_python//python/config_settings:python_version'
    major = sys.version_info.major
    minor = sys.version_info.minor
    bazel_build_args += [f'--{python_version_rule}={major}.{minor}']
    lib_extension = '.so'
    if platform.system() == 'Windows':
      # Required to build protobuf. See
      # https://github.com/protocolbuffers/protobuf/issues/12947
      bazel_startup_args += [r'--output_base=C:\O']
      # Needed by pybind11_bazel.
      os.environ['PYTHON_BIN_PATH'] = sys.executable
      os.environ['PYTHON_LIB_PATH'] = os.path.join(sys.exec_prefix, 'lib')
      lib_extension = '.pyd'

    if 'TINK_PYTHON_BAZEL_REMOTE_CACHE_GCS_BUCKET_URL' in os.environ:
      if 'TINK_PYTHON_BAZEL_REMOTE_CACHE_SERVICE_KEY_PATH' not in os.environ:
        raise ValueError(
            'TINK_PYTHON_BAZEL_REMOTE_CACHE_SERVICE_KEY_PATH must be set.'
        )
      remote_cache_gcs_bucket_url = os.environ[
          'TINK_PYTHON_BAZEL_REMOTE_CACHE_GCS_BUCKET_URL'
      ]
      remote_cache_gcs_service_key_path = os.path.abspath(
          os.environ['TINK_PYTHON_BAZEL_REMOTE_CACHE_SERVICE_KEY_PATH']
      )
      bazel_build_args += [
          f'--remote_cache={remote_cache_gcs_bucket_url}',
          f'--google_credentials={remote_cache_gcs_service_key_path}',
      ]

    # Run build command.
    self.spawn([self.bazel_command] + bazel_startup_args + bazel_build_args)

    ext_bazel_bin_path = os.path.join(
        'bazel-bin', ext.relpath, ext.target_name + lib_extension
    )
    ext_dest_path = self.get_ext_fullpath(ext.name)
    ext_dest_dir = os.path.dirname(ext_dest_path)
    if not os.path.exists(ext_dest_dir):
      os.makedirs(ext_dest_dir)
    shutil.copyfile(ext_bazel_bin_path, ext_dest_path)


def options() -> Mapping[str, Any]:
  bdist_wheel = {}
  if 'PLAT_NAME' in os.environ:
    bdist_wheel['plat_name'] = os.environ['PLAT_NAME']
  return {'bdist_wheel': bdist_wheel}


def main() -> None:
  # Generate compiled protocol buffers.
  protoc_command = _get_protoc_command()
  for proto_file in glob.glob('tink/proto/*.proto'):
    _generate_proto(protoc_command, proto_file)

  awskms_extra_requirements = _parse_requirements('requirements_awskms.in')
  gcpkms_extra_requirements = _parse_requirements('requirements_gcpkms.in')
  hcvault_extra_requirements = _parse_requirements('requirements_hcvault.in')

  setuptools.setup(
      name='tink',
      version=_get_tink_version(),
      url=f'{_TINK_CRYPTO_GITHUB_ORG_URL}/tink-py',
      description=(
          'A multi-language, cross-platform library that provides cryptographic'
          ' APIs that are secure, easy to use correctly, and hard(er) to'
          ' misuse.'
      ),
      author='Tink Developers',
      author_email='tink-users@googlegroups.com',
      long_description=open('README.md').read(),
      long_description_content_type='text/markdown',
      # Contained modules and scripts.
      packages=setuptools.find_packages(),
      install_requires=_parse_requirements('requirements.in'),
      extras_require={
          'gcpkms': gcpkms_extra_requirements,
          'awskms': awskms_extra_requirements,
          'hcvault': hcvault_extra_requirements,
          'all': (
              gcpkms_extra_requirements
              + awskms_extra_requirements
              + hcvault_extra_requirements
          ),
      },
      cmdclass=dict(build_ext=BuildBazelExtension),
      ext_modules=[
          BazelExtension('//tink/cc/pybind:tink_bindings'),
      ],
      zip_safe=False,
      # PyPI package information.
      classifiers=[
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
          'Programming Language :: Python :: 3.11',
          'Programming Language :: Python :: 3.12',
          'Programming Language :: Python :: 3.13',
          'Topic :: Software Development :: Libraries',
      ],
      options=options(),
      license='Apache 2.0',
      keywords='tink cryptography',
      python_requires='>=3.9',
  )


if __name__ == '__main__':
  main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.prf.prf_set_key_manager."""

from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import common_pb2
from tink.proto import hmac_prf_pb2
from tink.proto import tink_pb2
import tink
from tink import core
from tink import prf


def setUpModule():
  prf.register()


class PrfKeyManagerTest(parameterized.TestCase):

  def test_new_key_data_success(self):
    key_template = prf.prf_key_templates._create_hmac_key_template(
        key_size=32, hash_type=common_pb2.SHA256)
    key_manager = core.Registry.key_manager(key_template.type_url)
    key_data = key_manager.new_key_data(key_template)
    self.assertEqual(key_data.type_url, key_template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = hmac_prf_pb2.HmacPrfKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertEqual(key.params.hash, common_pb2.SHA256)
    self.assertLen(key.key_value, 32)

  def test_invalid_params_throw_exception(self):
    template = prf.prf_key_templates._create_hmac_key_template(
        key_size=7, hash_type=common_pb2.SHA256)
    with self.assertRaises(tink.TinkError):
      tink.new_keyset_handle(template)

  @parameterized.parameters([
      prf.prf_key_templates.AES_CMAC, prf.prf_key_templates.HMAC_SHA256,
      prf.prf_key_templates.HMAC_SHA512, prf.prf_key_templates.HKDF_SHA256
  ])
  def test_compute_success(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(prf.PrfSet)
    output = primitive.primary().compute(b'input_data', output_length=15)
    self.assertLen(output, 15)
    self.assertEqual(
        primitive.primary().compute(b'input_data', output_length=15), output)
    self.assertNotEqual(
        primitive.primary().compute(b'some_other_data', output_length=15),
        output)
    prfs = primitive.all()
    self.assertLen(prfs, 1)
    self.assertEqual(
        prfs[primitive.primary_id()].compute(b'input_data', output_length=15),
        output)

  @parameterized.parameters([
      prf.prf_key_templates.AES_CMAC, prf.prf_key_templates.HMAC_SHA256,
      prf.prf_key_templates.HMAC_SHA512, prf.prf_key_templates.HKDF_SHA256
  ])
  def test_output_too_long_raises_error(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(prf.PrfSet)
    with self.assertRaises(tink.TinkError):
      primitive.primary().compute(b'input_data', output_length=1234567)
    prfs = primitive.all()
    self.assertLen(prfs, 1)
    p = prfs[primitive.primary_id()]
    with self.assertRaises(tink.TinkError):
      p.compute(b'input_data', output_length=1234567)

if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.jwt._jwt_mac_wrapper."""

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import jwt_hmac_pb2
from tink.proto import tink_pb2
import tink
from tink import jwt
from tink import secret_key_access
from tink.jwt import _json_util
from tink.jwt import _jwt_format
from tink.testing import keyset_builder


def setUpModule():
  jwt.register_jwt_mac()


def _set_custom_kid(keyset_handle: tink.KeysetHandle,
                    custom_kid: str) -> tink.KeysetHandle:
  """Set the custom_kid field of the first key."""
  serialized_keyset = tink.proto_keyset_format.serialize(
      keyset_handle, secret_key_access.TOKEN
  )
  keyset = tink_pb2.Keyset.FromString(serialized_keyset)
  hmac_key = jwt_hmac_pb2.JwtHmacKey.FromString(keyset.key[0].key_data.value)
  hmac_key.custom_kid.value = custom_kid
  keyset.key[0].key_data.value = hmac_key.SerializeToString()
  return tink.proto_keyset_format.parse(
      keyset.SerializeToString(), secret_key_access.TOKEN
  )


def _change_key_id(keyset_handle: tink.KeysetHandle) -> tink.KeysetHandle:
  """Changes the key id of the first key and sets it primary."""
  serialized_keyset = tink.proto_keyset_format.serialize(
      keyset_handle, secret_key_access.TOKEN
  )
  keyset = tink_pb2.Keyset.FromString(serialized_keyset)
  # XOR the key id with an arbitrary 32-bit string to get a new key id.
  new_key_id = keyset.key[0].key_id ^ 0xdeadbeef
  keyset.key[0].key_id = new_key_id
  keyset.primary_key_id = new_key_id
  return tink.proto_keyset_format.parse(
      keyset.SerializeToString(), secret_key_access.TOKEN
  )


def _change_output_prefix_to_tink(
    keyset_handle: tink.KeysetHandle) -> tink.KeysetHandle:
  """Changes the output prefix type of the first key to TINK."""
  serialization = tink.proto_keyset_format.serialize(
      keyset_handle, secret_key_access.TOKEN
  )
  keyset = tink_pb2.Keyset.FromString(serialization)
  keyset.key[0].output_prefix_type = tink_pb2.TINK
  return tink.proto_keyset_format.parse(
      keyset.SerializeToString(), secret_key_access.TOKEN
  )


class JwtMacWrapperTest(parameterized.TestCase):

  @parameterized.parameters([
      (jwt.raw_jwt_hs256_template(), jwt.raw_jwt_hs256_template()),
      (jwt.raw_jwt_hs256_template(), jwt.jwt_hs256_template()),
      (jwt.jwt_hs256_template(), jwt.raw_jwt_hs256_template()),
      (jwt.jwt_hs256_template(), jwt.jwt_hs256_template()),
  ])
  def test_key_rotation(self, old_key_tmpl, new_key_tmpl):
    builder = keyset_builder.new_keyset_builder()
    older_key_id = builder.add_new_key(old_key_tmpl)

    builder.set_primary_key(older_key_id)
    jwtmac1 = builder.keyset_handle().primitive(jwt.JwtMac)

    newer_key_id = builder.add_new_key(new_key_tmpl)
    jwtmac2 = builder.keyset_handle().primitive(jwt.JwtMac)

    builder.set_primary_key(newer_key_id)
    jwtmac3 = builder.keyset_handle().primitive(jwt.JwtMac)

    builder.disable_key(older_key_id)
    jwtmac4 = builder.keyset_handle().primitive(jwt.JwtMac)

    raw_jwt = jwt.new_raw_jwt(issuer='a', without_expiration=True)
    validator = jwt.new_validator(
        expected_issuer='a', allow_missing_expiration=True)

    self.assertNotEqual(older_key_id, newer_key_id)
    # 1 uses the older key. So 1, 2 and 3 can verify the mac, but not 4.
    compact1 = jwtmac1.compute_mac_and_encode(raw_jwt)
    self.assertEqual(
        jwtmac1.verify_mac_and_decode(compact1, validator).issuer(), 'a')
    self.assertEqual(
        jwtmac2.verify_mac_and_decode(compact1, validator).issuer(), 'a')
    self.assertEqual(
        jwtmac3.verify_mac_and_decode(compact1, validator).issuer(), 'a')
    with self.assertRaises(tink.TinkError):
      jwtmac4.verify_mac_and_decode(compact1, validator)

    # 2 uses the older key. So 1, 2 and 3 can verify the mac, but not 4.
    compact2 = jwtmac2.compute_mac_and_encode(raw_jwt)
    self.assertEqual(
        jwtmac1.verify_mac_and_decode(compact2, validator).issuer(), 'a')
    self.assertEqual(
        jwtmac2.verify_mac_and_decode(compact2, validator).issuer(), 'a')
    self.assertEqual(
        jwtmac3.verify_mac_and_decode(compact2, validator).issuer(), 'a')
    with self.assertRaises(tink.TinkError):
      jwtmac4.verify_mac_and_decode(compact2, validator)

    # 3 uses the newer key. So 2, 3 and 4 can verify the mac, but not 1.
    compact3 = jwtmac3.compute_mac_and_encode(raw_jwt)
    with self.assertRaises(tink.TinkError):
      jwtmac1.verify_mac_and_decode(compact3, validator)
    self.assertEqual(
        jwtmac2.verify_mac_and_decode(compact3, validator).issuer(), 'a')
    self.assertEqual(
        jwtmac3.verify_mac_and_decode(compact3, validator).issuer(), 'a')
    self.assertEqual(
        jwtmac4.verify_mac_and_decode(compact3, validator).issuer(), 'a')

    # 4 uses the newer key. So 2, 3 and 4 can verify the mac, but not 1.
    compact4 = jwtmac4.compute_mac_and_encode(raw_jwt)
    with self.assertRaises(tink.TinkError):
      jwtmac1.verify_mac_and_decode(compact4, validator)
    self.assertEqual(
        jwtmac2.verify_mac_and_decode(compact4, validator).issuer(), 'a')
    self.assertEqual(
        jwtmac3.verify_mac_and_decode(compact4, validator).issuer(), 'a')
    self.assertEqual(
        jwtmac4.verify_mac_and_decode(compact4, validator).issuer(), 'a')

  def test_only_tink_output_prefix_type_encodes_a_kid_header(self):
    handle = tink.new_keyset_handle(jwt.raw_jwt_hs256_template())
    jwt_mac = handle.primitive(jwt.JwtMac)

    tink_handle = _change_output_prefix_to_tink(handle)
    tink_jwt_mac = tink_handle.primitive(jwt.JwtMac)

    raw_jwt = jwt.new_raw_jwt(issuer='issuer', without_expiration=True)

    token = jwt_mac.compute_mac_and_encode(raw_jwt)
    token_with_kid = tink_jwt_mac.compute_mac_and_encode(raw_jwt)

    _, header, _, _ = _jwt_format.split_signed_compact(token)
    self.assertNotIn('kid', _json_util.json_loads(header))

    _, header_with_kid, _, _ = _jwt_format.split_signed_compact(token_with_kid)
    self.assertIn('kid', _json_util.json_loads(header_with_kid))

    validator = jwt.new_validator(
        expected_issuer='issuer', allow_missing_expiration=True)
    jwt_mac.verify_mac_and_decode(token, validator)
    tink_jwt_mac.verify_mac_and_decode(token_with_kid, validator)

    # With output prefix type RAW, a kid header is ignored
    jwt_mac.verify_mac_and_decode(token_with_kid, validator)
    # With output prefix type TINK, a kid header is required.
    with self.assertRaises(tink.TinkError):
      tink_jwt_mac.verify_mac_and_decode(token, validator)

    other_handle = _change_key_id(tink_handle)
    other_jwt_mac = other_handle.primitive(jwt.JwtMac)
    # A token with a wrong kid is rejected, even if the signature is ok.
    with self.assertRaises(tink.TinkError):
      other_jwt_mac.verify_mac_and_decode(token_with_kid, validator)

  def test_raw_output_prefix_type_encodes_a_custom_kid_header(self):
    # normal HMAC jwt_mac with output prefix RAW
    handle = tink.new_keyset_handle(jwt.raw_jwt_hs256_template())
    raw_jwt = jwt.new_raw_jwt(issuer='issuer', without_expiration=True)
    validator = jwt.new_validator(
        expected_issuer='issuer', allow_missing_expiration=True)

    jwt_mac = handle.primitive(jwt.JwtMac)
    token = jwt_mac.compute_mac_and_encode(raw_jwt)
    jwt_mac.verify_mac_and_decode(token, validator)

    _, json_header, _, _ = _jwt_format.split_signed_compact(token)
    self.assertNotIn('kid', _json_util.json_loads(json_header))

    # HMAC jwt_mac with a custom_kid set
    custom_kid_handle = _set_custom_kid(handle, custom_kid='my kid')
    custom_kid_jwt_mac = custom_kid_handle.primitive(jwt.JwtMac)
    token_with_kid = custom_kid_jwt_mac.compute_mac_and_encode(raw_jwt)
    custom_kid_jwt_mac.verify_mac_and_decode(token_with_kid, validator)

    _, header_with_kid, _, _ = _jwt_format.split_signed_compact(token_with_kid)
    self.assertEqual(_json_util.json_loads(header_with_kid)['kid'], 'my kid')

    # Even when custom_kid is set, its not required to be set in the header.
    custom_kid_jwt_mac.verify_mac_and_decode(token, validator)
    # An additional kid header is ignored.
    jwt_mac.verify_mac_and_decode(token_with_kid, validator)

    other_handle = _set_custom_kid(handle, custom_kid='other kid')
    other_jwt_mac = other_handle.primitive(jwt.JwtMac)
    with self.assertRaises(tink.TinkError):
      # The custom_kid does not match the kid header.
      other_jwt_mac.verify_mac_and_decode(
          token_with_kid, validator)

    tink_handle = _change_output_prefix_to_tink(custom_kid_handle)
    tink_jwt_mac = tink_handle.primitive(jwt.JwtMac)
    # having custom_kid set with output prefix TINK is not allowed
    with self.assertRaises(tink.TinkError):
      tink_jwt_mac.compute_mac_and_encode(raw_jwt)
    with self.assertRaises(tink.TinkError):
      tink_jwt_mac.verify_mac_and_decode(token, validator)
    with self.assertRaises(tink.TinkError):
      tink_jwt_mac.verify_mac_and_decode(token_with_kid, validator)

  def test_legacy_key_fails(self):
    template = keyset_builder.legacy_template(jwt.raw_jwt_hs256_template())
    builder = keyset_builder.new_keyset_builder()
    key_id = builder.add_new_key(template)
    builder.set_primary_key(key_id)
    handle = builder.keyset_handle()
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtMac)

  def test_legacy_non_primary_key_fails(self):
    builder = keyset_builder.new_keyset_builder()
    old_template = keyset_builder.legacy_template(jwt.raw_jwt_hs256_template())
    _ = builder.add_new_key(old_template)
    current_key_id = builder.add_new_key(jwt.jwt_hs256_template())
    builder.set_primary_key(current_key_id)
    handle = builder.keyset_handle()
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtMac)

  def test_jwt_mac_from_keyset_without_primary_fails(self):
    builder = keyset_builder.new_keyset_builder()
    builder.add_new_key(jwt.raw_jwt_hs256_template())
    with self.assertRaises(tink.TinkError):
      builder.keyset_handle()


if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Functions that help to serialize and deserialize from/to the JWT format."""

import base64
import binascii
import struct
from typing import Any, Optional, Tuple

from tink.proto import tink_pb2
from tink.jwt import _json_util
from tink.jwt import _jwt_error
from tink.jwt import _raw_jwt

_VALID_ALGORITHMS = frozenset({
    'HS256', 'HS384', 'HS512', 'ES256', 'ES384', 'ES512', 'RS256', 'RS384',
    'RS384', 'RS512', 'PS256', 'PS384', 'PS512'
})


def base64_encode(data: bytes) -> bytes:
  """Does a URL-safe base64 encoding without padding."""
  return base64.urlsafe_b64encode(data).rstrip(b'=')


def _is_valid_urlsafe_base64_char(c: int) -> bool:
  if c >= ord('a') and c <= ord('z'):
    return True
  if c >= ord('A') and c <= ord('Z'):
    return True
  if c >= ord('0') and c <= ord('9'):
    return True
  if c == ord('-') or c == ord('_'):
    return True
  return False


def base64_decode(encoded_data: bytes) -> bytes:
  """Does a URL-safe base64 decoding without padding."""
  # base64.urlsafe_b64decode ignores all non-base64 chars. We don't want that.
  for c in encoded_data:
    if not _is_valid_urlsafe_base64_char(c):
      raise _jwt_error.JwtInvalidError('invalid base64 encoding')
  # base64.urlsafe_b64decode requires padding, but does not mind too much
  # padding. So we simply add the maximum amount of padding needed.
  padded_encoded_data = encoded_data + b'==='
  try:
    return base64.urlsafe_b64decode(padded_encoded_data)
  except binascii.Error:
    # Throws when the length of encoded_data is (4*i + 1) for some i
    raise _jwt_error.JwtInvalidError('invalid base64 encoding')


def _validate_algorithm(algorithm: str) -> None:
  if algorithm not in _VALID_ALGORITHMS:
    raise _jwt_error.JwtInvalidError('Invalid algorithm %s' % algorithm)


def encode_header(json_header: str) -> bytes:
  try:
    return base64_encode(json_header.encode('utf8'))
  except UnicodeEncodeError:
    raise _jwt_error.JwtInvalidError('invalid token')


def decode_header(encoded_header: bytes) -> str:
  try:
    return base64_decode(encoded_header).decode('utf8')
  except UnicodeDecodeError:
    raise _jwt_error.JwtInvalidError('invalid token')


def encode_payload(json_payload: str) -> bytes:
  """Encodes the payload into compact form."""
  try:
    return base64_encode(json_payload.encode('utf8'))
  except UnicodeEncodeError:
    raise _jwt_error.JwtInvalidError('invalid token')


def decode_payload(encoded_payload: bytes) -> str:
  """Decodes the payload from compact form."""
  try:
    return base64_decode(encoded_payload).decode('utf8')
  except UnicodeDecodeError:
    raise _jwt_error.JwtInvalidError('invalid token')


def encode_signature(signature: bytes) -> bytes:
  """Encodes the signature."""
  return base64_encode(signature)


def decode_signature(encoded_signature: bytes) -> bytes:
  """Decodes the signature."""
  return base64_decode(encoded_signature)


def create_header(algorithm: str, type_header: Optional[str],
                  kid: Optional[str]) -> bytes:
  _validate_algorithm(algorithm)
  header = {}
  if kid:
    header['kid'] = kid
  header['alg'] = algorithm
  if type_header:
    header['typ'] = type_header
  return encode_header(_json_util.json_dumps(header))


def get_kid(key_id: int, prefix: tink_pb2.OutputPrefixType) -> Optional[str]:
  """Returns the encoded key_id, or None."""
  if prefix == tink_pb2.RAW:
    return None
  if prefix == tink_pb2.TINK:
    if key_id < 0 or key_id > 2**32:
      raise _jwt_error.JwtInvalidError('invalid key_id')
    return base64_encode(struct.pack('>L', key_id)).decode('utf8')
  raise _jwt_error.JwtInvalidError('unexpected output prefix type')


def split_signed_compact(signed_compact: str) -> Tuple[bytes, str, str, bytes]:
  """Splits a signed compact into its parts.

  Args:
    signed_compact: A signed compact JWT.

  Returns:
    A (unsigned_compact, json_header, json_payload, signature_or_mac) tuple.
  Raises:
    _jwt_error.JwtInvalidError if it fails.
  """
  if not isinstance(signed_compact, str):
    raise _jwt_error.JwtInvalidError('invalid token: not a str')
  try:
    encoded = signed_compact.encode('utf8')
  except UnicodeEncodeError:
    raise _jwt_error.JwtInvalidError('invalid token')
  try:
    unsigned_compact, encoded_signature = encoded.rsplit(b'.', 1)
  except ValueError:
    raise _jwt_error.JwtInvalidError('invalid token')
  signature_or_mac = decode_signature(encoded_signature)
  try:
    encoded_header, encoded_payload = unsigned_compact.split(b'.')
  except ValueError:
    raise _jwt_error.JwtInvalidError('invalid token')

  json_header = decode_header(encoded_header)
  json_payload = decode_payload(encoded_payload)
  return (unsigned_compact, json_header, json_payload, signature_or_mac)


def _validate_kid_header(header: Any, kid: str) -> None:
  if header['kid'] != kid:
    raise _jwt_error.JwtInvalidError('invalid kid header')


def validate_header(header: Any,
                    algorithm: str,
                    tink_kid: Optional[str] = None,
                    custom_kid: Optional[str] = None) -> None:
  """Parses the header and validates its values."""
  _validate_algorithm(algorithm)
  hdr_algorithm = header.get('alg', '')
  if hdr_algorithm.upper() != algorithm:
    raise _jwt_error.JwtInvalidError('Invalid algorithm; expected %s, got %s' %
                                     (algorithm, hdr_algorithm))
  if 'crit' in header:
    raise _jwt_error.JwtInvalidError(
        'all tokens with crit headers are rejected')
  if tink_kid is not None and custom_kid is not None:
    raise _jwt_error.JwtInvalidError('custom_kid can only be set for RAW keys')
  if tink_kid is not None:
    if 'kid' not in header:
      # for output prefix type TINK, the kid header is required
      raise _jwt_error.JwtInvalidError('missing kid in header')
    _validate_kid_header(header, tink_kid)
  if custom_kid is not None and 'kid' in header:
    _validate_kid_header(header, custom_kid)


def get_type_header(header: Any) -> Optional[str]:
  return header.get('typ', None)


def create_unsigned_compact(algorithm: str, kid: Optional[str],
                            raw_jwt: _raw_jwt.RawJwt) -> bytes:
  if raw_jwt.has_type_header():
    header = create_header(algorithm, raw_jwt.type_header(), kid)
  else:
    header = create_header(algorithm, None, kid)
  return header + b'.' + encode_payload(raw_jwt.json_payload())


def create_signed_compact(unsigned_compact: bytes, signature: bytes) -> str:
  return (unsigned_compact + b'.' + encode_signature(signature)).decode('utf8')



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.jwt._jwt_signature_key_manager."""

import datetime

from typing import cast

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import jwt_ecdsa_pb2
from tink.proto import tink_pb2
import tink
from tink import jwt
from tink.jwt import _jwt_format

from tink.jwt import _jwt_signature_key_manager
from tink.jwt import _jwt_signature_wrappers

DATETIME_1970 = datetime.datetime.fromtimestamp(12345, datetime.timezone.utc)
DATETIME_2011 = datetime.datetime.fromtimestamp(1300819380,
                                                datetime.timezone.utc)
DATETIME_2020 = datetime.datetime.fromtimestamp(1582230020,
                                                datetime.timezone.utc)


def setUpModule():
  jwt.register_jwt_signature()


def gen_compact(json_header: str, json_payload: str, raw_sign) -> str:
  unsigned_compact = (
      _jwt_format.encode_header(json_header) + b'.' +
      _jwt_format.encode_payload(json_payload))
  signature = raw_sign.sign(unsigned_compact)
  return _jwt_format.create_signed_compact(unsigned_compact, signature)


class JwtSignatureKeyManagerTest(parameterized.TestCase):

  def test_create_sign_verify(self):
    handle = tink.new_keyset_handle(jwt.jwt_es256_template())
    sign = handle.primitive(jwt.JwtPublicKeySign)
    verify = handle.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)
    raw_jwt = jwt.new_raw_jwt(
        issuer='joe',
        expiration=DATETIME_2011,
        custom_claims={'http://example.com/is_root': True})
    signed_compact = sign.sign_and_encode(raw_jwt)

    validator = jwt.new_validator(
        expected_issuer='joe', fixed_now=DATETIME_1970)
    verified_jwt = verify.verify_and_decode(signed_compact, validator)
    self.assertEqual(verified_jwt.issuer(), 'joe')
    self.assertEqual(verified_jwt.expiration().year, 2011)
    self.assertCountEqual(verified_jwt.custom_claim_names(),
                          ['http://example.com/is_root'])
    self.assertTrue(verified_jwt.custom_claim('http://example.com/is_root'))

    # fails because it is expired
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(
          signed_compact,
          jwt.new_validator(expected_issuer='joe', fixed_now=DATETIME_2020))

    # wrong issuer
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(
          signed_compact,
          jwt.new_validator(expected_issuer='jane', fixed_now=DATETIME_1970))

    # invalid format
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(signed_compact + '.123', validator)

    # invalid character
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(signed_compact + '?', validator)

    # modified signature
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(signed_compact + 'a', validator)

    # modified header
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode('a' + signed_compact, validator)

  def test_create_sign_verify_with_type_header(self):
    handle = tink.new_keyset_handle(jwt.jwt_es256_template())
    sign = handle.primitive(jwt.JwtPublicKeySign)
    verify = handle.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)
    raw_jwt = jwt.new_raw_jwt(
        type_header='typeHeader', issuer='joe', without_expiration=True)
    signed_compact = sign.sign_and_encode(raw_jwt)

    validator = jwt.new_validator(
        expected_type_header='typeHeader',
        expected_issuer='joe',
        allow_missing_expiration=True)
    verified_jwt = verify.verify_and_decode(signed_compact, validator)
    self.assertEqual(verified_jwt.type_header(), 'typeHeader')

  def test_verify_with_other_key_fails(self):
    handle = tink.new_keyset_handle(jwt.jwt_es256_template())
    sign = handle.primitive(jwt.JwtPublicKeySign)
    raw_jwt = jwt.new_raw_jwt(issuer='issuer', without_expiration=True)
    compact = sign.sign_and_encode(raw_jwt)

    other_handle = tink.new_keyset_handle(jwt.jwt_es256_template())
    other_verify = other_handle.public_keyset_handle().primitive(
        jwt.JwtPublicKeyVerify)
    with self.assertRaises(tink.TinkError):
      other_verify.verify_and_decode(
          compact,
          jwt.new_validator(
              expected_issuer='issuer', allow_missing_expiration=True))

  def test_weird_tokens_with_valid_signatures(self):
    handle = tink.new_keyset_handle(jwt.raw_jwt_es256_template())
    sign = handle.primitive(jwt.JwtPublicKeySign)
    # Get the internal PublicKeySign primitive to create valid signatures.
    wrapped = cast(_jwt_signature_wrappers._WrappedJwtPublicKeySign, sign)
    raw_sign = cast(_jwt_signature_key_manager._JwtPublicKeySign,
                    wrapped._primitive_set.primary().primitive)._public_key_sign

    verify = handle.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)
    validator = jwt.new_validator(
        expected_issuer='issuer', allow_missing_expiration=True)

    # Normal token.
    valid = gen_compact('{"alg":"ES256"}', '{"iss":"issuer"}', raw_sign)
    verified_jwt = verify.verify_and_decode(valid, validator)
    self.assertEqual(verified_jwt.issuer(), 'issuer')

    # Token with unknown header is valid.
    unknown_header = gen_compact('{"alg":"ES256","unknown_header":"abc"} \n ',
                                 '{"iss":"issuer" }', raw_sign)
    verified_jwt = verify.verify_and_decode(unknown_header, validator)
    self.assertEqual(verified_jwt.issuer(), 'issuer')

    # Token with unknown kid is valid, since primitives with output prefix type
    # RAW ignore kid headers.
    unknown_header = gen_compact('{"alg":"ES256","kid":"unknown"} \n ',
                                 '{"iss":"issuer" }', raw_sign)
    verified_jwt = verify.verify_and_decode(unknown_header, validator)
    self.assertEqual(verified_jwt.issuer(), 'issuer')

    # Token with invalid alg header
    alg_invalid = gen_compact('{"alg":"ES384"}', '{"iss":"issuer"}', raw_sign)
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(alg_invalid, validator)

    # Token with empty header
    empty_header = gen_compact('{}', '{"iss":"issuer"}', raw_sign)
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(empty_header, validator)

    # Token header is not valid JSON
    header_invalid = gen_compact('{"alg":"ES256"', '{"iss":"issuer"}', raw_sign)
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(header_invalid, validator)

    # Token payload is not valid JSON
    payload_invalid = gen_compact('{"alg":"ES256"}', '{"iss":"issuer"',
                                  raw_sign)
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(payload_invalid, validator)

    # Token with whitespace in header JSON string is valid.
    whitespace_in_header = gen_compact(' {"alg":   \n  "ES256"} \n ',
                                       '{"iss":"issuer" }', raw_sign)
    verified_jwt = verify.verify_and_decode(whitespace_in_header, validator)
    self.assertEqual(verified_jwt.issuer(), 'issuer')

    # Token with whitespace in payload JSON string is valid.
    whitespace_in_payload = gen_compact('{"alg":"ES256"}',
                                        ' {"iss": \n"issuer" } \n', raw_sign)
    verified_jwt = verify.verify_and_decode(whitespace_in_payload, validator)
    self.assertEqual(verified_jwt.issuer(), 'issuer')

    # Token with whitespace in base64-encoded header is invalid.
    with_whitespace = (
        _jwt_format.encode_header('{"alg":"ES256"}') + b' .' +
        _jwt_format.encode_payload('{"iss":"issuer"}'))
    token_with_whitespace = _jwt_format.create_signed_compact(
        with_whitespace, raw_sign.sign(with_whitespace))
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(token_with_whitespace, validator)

    # Token with invalid character is invalid.
    with_invalid_char = (
        _jwt_format.encode_header('{"alg":"ES256"}') + b'.?' +
        _jwt_format.encode_payload('{"iss":"issuer"}'))
    token_with_invalid_char = _jwt_format.create_signed_compact(
        with_invalid_char, raw_sign.sign(with_invalid_char))
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(token_with_invalid_char, validator)

    # Token with additional '.' is invalid.
    with_dot = (
        _jwt_format.encode_header('{"alg":"ES256"}') + b'.' +
        _jwt_format.encode_payload('{"iss":"issuer"}') + b'.')
    token_with_dot = _jwt_format.create_signed_compact(
        with_dot, raw_sign.sign(with_dot))
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(token_with_dot, validator)

    # num_recursions has been chosen such that parsing of this token fails
    # in all languages. We want to make sure that the algorithm does not
    # hang or crash in this case, but only returns a parsing error.
    num_recursions = 10000
    rec_payload = ('{"a":' * num_recursions) + '""' + ('}' * num_recursions)
    rec_token = gen_compact('{"alg":"ES256"}', rec_payload, raw_sign)
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(
          rec_token, validator=jwt.new_validator(allow_missing_expiration=True))

    # test wrong types
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(cast(str, None), validator)
    with self.assertRaises(tink.TinkError):
      verify.verify_and_decode(cast(str, 123), validator)
    with self.assertRaises(tink.TinkError):
      valid_bytes = valid.encode('utf8')
      verify.verify_and_decode(cast(str, valid_bytes), validator)

  def test_create_ecdsa_handle_with_invalid_algorithm_fails(self):
    key_format = jwt_ecdsa_pb2.JwtEcdsaKeyFormat(
        algorithm=jwt_ecdsa_pb2.ES_UNKNOWN)
    template = tink_pb2.KeyTemplate(
        type_url='type.googleapis.com/google.crypto.tink.JwtEcdsaPrivateKey',
        value=key_format.SerializeToString(),
        output_prefix_type=tink_pb2.RAW)
    with self.assertRaises(tink.TinkError):
      tink.new_keyset_handle(template)

  def test_create_sign_primitive_with_invalid_algorithm_fails(self):
    handle = tink.new_keyset_handle(jwt.jwt_es256_template())
    key = jwt_ecdsa_pb2.JwtEcdsaPrivateKey.FromString(
        handle._keyset.key[0].key_data.value)
    key.public_key.algorithm = jwt_ecdsa_pb2.ES_UNKNOWN
    handle._keyset.key[0].key_data.value = key.SerializeToString()
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtPublicKeySign)

  def test_create_verify_primitive_with_invalid_algorithm_fails(self):
    private_handle = tink.new_keyset_handle(jwt.jwt_es256_template())
    handle = private_handle.public_keyset_handle()
    key = jwt_ecdsa_pb2.JwtEcdsaPublicKey.FromString(
        handle._keyset.key[0].key_data.value)
    key.algorithm = jwt_ecdsa_pb2.ES_UNKNOWN
    handle._keyset.key[0].key_data.value = key.SerializeToString()
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtPublicKeyVerify)


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.streaming_aead.encrypting_stream."""

import io
from typing import cast

from absl.testing import absltest

from tink import core
from tink import streaming_aead
from tink.streaming_aead import _raw_streaming_aead

# Using malformed UTF-8 sequences to ensure there is no accidental decoding.
B_X80 = b'\x80'
B_AAD_ = b'aa' + B_X80
B_ASSOC_ = b'asso' + B_X80


def setUpModule():
  streaming_aead.register()


def get_raw_primitive():
  key_data = core.Registry.new_key_data(
      streaming_aead.streaming_aead_key_templates.AES128_CTR_HMAC_SHA256_4KB)
  return core.Registry.primitive(key_data, _raw_streaming_aead.RawStreamingAead)


class EncryptingStreamTest(absltest.TestCase):

  def test_write_non_bytes(self):
    f = io.BytesIO()
    with get_raw_primitive().new_raw_encrypting_stream(f, B_AAD_) as es:
      with self.assertRaisesRegex(TypeError, 'bytes-like object is required'):
        es.write(cast(bytes, 'This is a string, not a bytes object'))

  def test_flush(self):
    f = io.BytesIO()
    with get_raw_primitive().new_raw_encrypting_stream(f, B_ASSOC_) as es:
      es.write(b'Hello world!' + B_X80)
      es.flush()

  def test_closed(self):
    f = io.BytesIO()
    es = get_raw_primitive().new_raw_encrypting_stream(f, B_ASSOC_)
    es.write(b'Hello world!' + B_X80)
    es.close()

    self.assertTrue(es.closed)
    self.assertTrue(f.closed)

  def test_closed_methods_raise(self):
    f = io.BytesIO()
    es = get_raw_primitive().new_raw_encrypting_stream(f, B_ASSOC_)
    es.write(b'Hello world!' + B_X80)
    es.close()

    with self.assertRaisesRegex(ValueError, 'closed'):
      es.write(b'Goodbye world.' + B_X80)
    with self.assertRaisesRegex(ValueError, 'closed'):
      with es:
        pass
    with self.assertRaisesRegex(ValueError, 'closed'):
      es.flush()

  def test_unsupported_operation(self):
    f = io.BytesIO()
    with get_raw_primitive().new_raw_encrypting_stream(f, B_ASSOC_) as es:
      with self.assertRaises(io.UnsupportedOperation):
        es.seek(0, 2)
      with self.assertRaises(io.UnsupportedOperation):
        es.truncate(0)
      with self.assertRaises(io.UnsupportedOperation):
        es.read(-1)

  def test_inquiries(self):
    f = io.BytesIO()
    with get_raw_primitive().new_raw_encrypting_stream(f, B_ASSOC_) as es:
      self.assertTrue(es.writable())
      self.assertFalse(es.readable())
      self.assertFalse(es.seekable())

  def test_context_manager_exception_closes_dest_file(self):
    """Tests that exceptional exits trigger normal file closure.

    Any other behaviour seems to be difficult to implement, since standard
    file wrappers (such as io.BufferedWriter, or io.TextIOWrapper) will always
    close the wrapped file, even if an error was raised.
    """
    ciphertext_destination = io.BytesIO()
    with self.assertRaisesRegex(ValueError, 'raised inside'):
      with get_raw_primitive().new_raw_encrypting_stream(
          ciphertext_destination, B_ASSOC_) as es:
        es.write(b'some message' + B_X80)
        raise ValueError('Error raised inside context manager')
    self.assertTrue(ciphertext_destination.closed)


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A file-like object that decrypts the data it reads.

It reads the ciphertext from a given other file-like object, and decrypts it.
"""

import io
from typing import BinaryIO, Optional

from tink import core
from tink.cc.pybind import tink_bindings
from tink.streaming_aead import _file_object_adapter


@core.use_tink_errors
def _get_input_stream_adapter(cc_primitive, aad, source):
  """Implemented as a separate method to ensure correct error transform."""
  return tink_bindings.new_cc_decrypting_stream(
      cc_primitive, aad, source)


class RawDecryptingStream(io.RawIOBase):
  """A file-like object which decrypts reads from an underlying object.

  It reads the ciphertext from the wrapped file-like object, and decrypts it.
  """

  def __init__(self, stream_aead: tink_bindings.StreamingAead,
               ciphertext_source: BinaryIO, associated_data: bytes, *,
               close_ciphertext_source: bool):
    """Create a new RawDecryptingStream.

    Args:
      stream_aead: C++ StreamingAead primitive from which a C++ DecryptingStream
        will be obtained.
      ciphertext_source: A readable file-like object from which ciphertext bytes
        will be read.
      associated_data: The associated data to use for decryption.
      close_ciphertext_source: Whether ciphertext_source should be closed when
        close() is called.
    """
    super().__init__()
    self._ciphertext_source = ciphertext_source
    self._close_ciphertext_source = close_ciphertext_source
    if not ciphertext_source.readable():
      raise ValueError('ciphertext_source must be readable')
    cc_ciphertext_source = _file_object_adapter.FileObjectAdapter(
        ciphertext_source)
    self._input_stream_adapter = _get_input_stream_adapter(
        stream_aead, associated_data, cc_ciphertext_source)

  @core.use_tink_errors
  def _read_from_input_stream_adapter(self, size: int) -> bytes:
    """Implemented as a separate method to ensure correct error transform."""
    return self._input_stream_adapter.read(size)

  def read(self, size: Optional[int] = -1) -> bytes:
    """Read and return up to size bytes, where size is an int.

    It blocks until at least one byte can be returned.

    Args:
      size: Maximum number of bytes to read. As a convenience, if size is
      unspecified or -1, all bytes until EOF are returned.

    Returns:
      Bytes read. If b'' is returned and size was not 0, this indicates EOF.

    Raises:
      TinkError if there was a permanent error.
    """
    if self.closed:  # pylint:disable=using-constant-test
      raise ValueError('read on closed file.')
    if size is None:
      size = -1
    if size < 0:
      return self.readall()
    try:
      # _input_stream_adapter may return an empty string when there is currently
      # no data is available. In Python (in blocking mode), read is expected to
      # block until some data is available.
      # https://docs.python.org/3/library/io.html also mentions a
      # non-blocking mode, but according to https://bugs.python.org/issue13322
      # that mode is not properly implemented and not really used.
      while True:
        data = self._read_from_input_stream_adapter(size)
        if data:
          return data
    except tink_bindings.PythonTinkStreamFinishedException:
      return b''

  def readinto(self, b: bytearray) -> int:
    """Read bytes into a pre-allocated bytes-like object b.

    Args:
      b: Bytes-like object to which data will be read.

    Returns:
      Number of bytes read. It returns 0 if EOF is reached, and None if no data
      is available at the moment.

    Raises:
      TinkError if there was a permanent error.
    """
    data = self.read(len(b))
    n = len(data)
    b[:n] = data
    return n

  def close(self) -> None:
    """Close the stream. Has no effect on a closed stream."""
    if self.closed:  # pylint:disable=using-constant-test
      return
    if self._close_ciphertext_source:
      self._ciphertext_source.close()
    super().close()

  def readable(self) -> bool:
    """Return True if the stream can be read from."""
    return True

  # b has type "Buffer", which is not yet supported by pytype.
  def write(self, b) -> Optional[int]:
    raise io.UnsupportedOperation()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for secret_key_access."""

from absl.testing import absltest

import tink
from tink import secret_key_access


class OtherKeyAccess(tink.KeyAccess):
  pass


class SecretKeyAccessTest(absltest.TestCase):

  def test_has_secret_access(self):
    self.assertTrue(
        tink.has_secret_key_access(secret_key_access.TOKEN))

  def test_public_key_access_does_not_have_secret_access(self):
    self.assertFalse(
        tink.has_secret_key_access(tink.PUBLIC_KEY_ACCESS_TOKEN))

  def test_other_key_access_does_not_have_secret_access(self):
    other_token = OtherKeyAccess()
    self.assertFalse(tink.has_secret_key_access(other_token))


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""MAC wrapper."""

from typing import Optional, Type

from tink.proto import tink_pb2
from tink import _monitoring
from tink import core
from tink.mac import _mac


class _WrappedMac(_mac.Mac):
  """Implements Mac for a set of Mac primitives."""

  def __init__(
      self,
      pset: core.PrimitiveSet,
      compute_monitor: Optional[_monitoring.KeyUsageMonitor] = None,
      verify_monitor: Optional[_monitoring.KeyUsageMonitor] = None,
  ):
    self._primitive_set = pset
    self._compute_monitor = compute_monitor
    self._verify_monitor = verify_monitor

  def compute_mac(self, data: bytes) -> bytes:
    primary = self._primitive_set.primary()
    if primary.output_prefix_type == tink_pb2.LEGACY:
      result = primary.identifier + primary.primitive.compute_mac(
          data + core.crypto_format.LEGACY_START_BYTE
      )
    else:
      result = primary.identifier + primary.primitive.compute_mac(data)

    if self._compute_monitor:
      self._compute_monitor.log(primary.key_id, len(data))

    return result

  def verify_mac(self, mac_value: bytes, data: bytes) -> None:
    if len(mac_value) <= core.crypto_format.NON_RAW_PREFIX_SIZE:
      if self._verify_monitor:
        self._verify_monitor.log_failure()
      # This also rejects raw MAC with size of 4 bytes or fewer. Those MACs are
      # clearly insecure, thus should be discouraged.
      raise core.TinkError('tag too short')
    prefix = mac_value[: core.crypto_format.NON_RAW_PREFIX_SIZE]
    mac_no_prefix = mac_value[core.crypto_format.NON_RAW_PREFIX_SIZE :]
    for entry in self._primitive_set.primitive_from_identifier(prefix):
      try:
        if entry.output_prefix_type == tink_pb2.LEGACY:
          entry.primitive.verify_mac(mac_no_prefix, data + b'\x00')
        else:
          entry.primitive.verify_mac(mac_no_prefix, data)

        # If there is no exception, the MAC is valid and we can return.
        if self._verify_monitor:
          self._verify_monitor.log(entry.key_id, len(mac_no_prefix))
        return
      except core.TinkError:
        pass

    # No 'non-raw' key matched, so let's try the raw keys (if any exist).
    for entry in self._primitive_set.raw_primitives():
      try:
        entry.primitive.verify_mac(mac_value, data)

        # If there is no exception, the MAC is valid and we can return.
        if self._verify_monitor:
          self._verify_monitor.log(entry.key_id, len(mac_value))
        return
      except core.TinkError:
        pass

    if self._verify_monitor:
      self._verify_monitor.log_failure()

    raise core.TinkError('invalid MAC')


class MacWrapper(core.PrimitiveWrapper[_mac.Mac, _mac.Mac]):
  """MacWrapper is the implementation of PrimitiveWrapper for the Mac primitive.

  The returned primitive works with a keyset (rather than a single key). To
  compute a MAC tag, it uses the primary key in the keyset, and prepends to the
  tag a certain prefix associated with the primary key. To verify a tag, the
  primitive uses the prefix of the tag to efficiently select the right key in
  the set. If the keys associated with the prefix do not validate the tag, the
  primitive tries all keys with tink_pb2.OutputPrefixType = tink_pb2.RAW.
  """

  def wrap(self, pset: core.PrimitiveSet) -> _mac.Mac:
    return _WrappedMac(pset)

  def primitive_class(self) -> Type[_mac.Mac]:
    return _mac.Mac

  def input_primitive_class(self) -> Type[_mac.Mac]:
    return _mac.Mac

  def _wrap_with_monitoring_info(
      self,
      pset: core.PrimitiveSet,
      monitoring_keyset_info: _monitoring.MonitoringKeySetInfo,
  ) -> _mac.Mac:
    compute_key_usage_monitor = _monitoring.get_key_usage_monitor_or_none(
        _monitoring.MonitoringContext(
            primitive='mac',
            api_function='compute_mac',
            keyset_info=monitoring_keyset_info,
        )
    )
    verify_key_usage_monitor = _monitoring.get_key_usage_monitor_or_none(
        _monitoring.MonitoringContext(
            primitive='mac',
            api_function='verify_mac',
            keyset_info=monitoring_keyset_info,
        )
    )
    return _WrappedMac(
        pset, compute_key_usage_monitor, verify_key_usage_monitor
    )



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.streaming_aead_key_manager."""

import io

from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import aes_ctr_hmac_streaming_pb2
from tink.proto import aes_gcm_hkdf_streaming_pb2
from tink.proto import common_pb2
from tink.proto import tink_pb2
import tink
from tink import core
from tink import streaming_aead
from tink.streaming_aead import _raw_streaming_aead
from tink.testing import bytes_io

# Using malformed UTF-8 sequences to ensure there is no accidental decoding.
B_X80 = b'\x80'


def setUpModule():
  streaming_aead.register()


def new_raw_primitive():
  key_data = core.Registry.new_key_data(
      streaming_aead.streaming_aead_key_templates
      .AES128_CTR_HMAC_SHA256_4KB)
  return core.Registry.primitive(key_data,
                                 _raw_streaming_aead.RawStreamingAead)


class StreamingAeadKeyManagerTest(parameterized.TestCase):

  def test_new_aes_gcm_hkdf_key_data(self):
    key_template = (
        streaming_aead.streaming_aead_key_templates.AES128_GCM_HKDF_4KB)
    key_data = core.Registry.new_key_data(key_template)
    self.assertEqual(key_data.type_url, key_template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = aes_gcm_hkdf_streaming_pb2.AesGcmHkdfStreamingKey.FromString(
        key_data.value)
    self.assertEqual(key.version, 0)
    self.assertLen(key.key_value, 16)
    self.assertEqual(key.params.hkdf_hash_type, common_pb2.HashType.SHA256)
    self.assertEqual(key.params.derived_key_size, 16)
    self.assertEqual(key.params.ciphertext_segment_size, 4096)

  def test_new_aes_ctr_hmac_key_data(self):
    key_template = (
        streaming_aead.streaming_aead_key_templates.AES128_CTR_HMAC_SHA256_4KB)
    key_data = core.Registry.new_key_data(key_template)
    self.assertEqual(key_data.type_url, key_template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = aes_ctr_hmac_streaming_pb2.AesCtrHmacStreamingKey.FromString(
        key_data.value)
    self.assertEqual(key.version, 0)
    self.assertLen(key.key_value, 16)
    self.assertEqual(key.params.hkdf_hash_type, common_pb2.HashType.SHA256)
    self.assertEqual(key.params.derived_key_size, 16)
    self.assertEqual(key.params.hmac_params.hash, common_pb2.HashType.SHA256)
    self.assertEqual(key.params.hmac_params.tag_size, 32)
    self.assertEqual(key.params.ciphertext_segment_size, 4096)

  def test_invalid_aes_gcm_hkdf_params_throw_exception(self):
    tmpls = streaming_aead.streaming_aead_key_templates
    key_template = tmpls.create_aes_gcm_hkdf_streaming_key_template(
        63, common_pb2.HashType.SHA1, 65, 55)
    with self.assertRaisesRegex(core.TinkError,
                                'key_size must not be smaller than'):
      core.Registry.new_key_data(key_template)

  def test_invalid_aes_ctr_hmac_params_throw_exception(self):
    tmpls = streaming_aead.streaming_aead_key_templates
    key_template = tmpls.create_aes_ctr_hmac_streaming_key_template(
        63, common_pb2.HashType.SHA1, 65, common_pb2.HashType.SHA256, 55, 2)
    with self.assertRaisesRegex(core.TinkError,
                                'key_size must not be smaller than'):
      core.Registry.new_key_data(key_template)

  def test_raw_encrypt_decrypt_readall(self):
    raw_primitive = new_raw_primitive()
    plaintext = b'plaintext' + B_X80
    aad = b'associated_data' + B_X80

    # Encrypt
    ct_destination = bytes_io.BytesIOWithValueAfterClose()
    with raw_primitive.new_raw_encrypting_stream(ct_destination, aad) as es:
      self.assertLen(plaintext, es.write(plaintext))
    # context manager closes es, which also closes ciphertext_dest
    self.assertTrue(ct_destination.closed)

    # Decrypt, with and without close_ciphertext_source
    for close_ciphertext_source in [True, False]:
      ct_source = io.BytesIO(ct_destination.value_after_close())
      with raw_primitive.new_raw_decrypting_stream(
          ct_source, aad,
          close_ciphertext_source=close_ciphertext_source) as ds:
        output = ds.readall()
      self.assertEqual(ct_source.closed, close_ciphertext_source)
      self.assertEqual(output, plaintext)

  def test_raw_encrypt_decrypt_read(self):
    raw_primitive = new_raw_primitive()
    plaintext = b'plaintext'
    aad = b'aad'

    ct_destination = bytes_io.BytesIOWithValueAfterClose()
    with raw_primitive.new_raw_encrypting_stream(ct_destination, aad) as es:
      es.write(plaintext)

    ct_source = io.BytesIO(ct_destination.value_after_close())
    with raw_primitive.new_raw_decrypting_stream(
        ct_source, aad, close_ciphertext_source=True) as ds:
      self.assertEqual(ds.read(5), b'plain')
      self.assertEqual(ds.read(5), b'text')

  def test_raw_encrypt_decrypt_readinto(self):
    raw_primitive = new_raw_primitive()
    plaintext = b'plaintext'
    aad = b'aad'

    ct_destination = bytes_io.BytesIOWithValueAfterClose()
    with raw_primitive.new_raw_encrypting_stream(ct_destination, aad) as es:
      es.write(plaintext)

    ct_source = io.BytesIO(ct_destination.value_after_close())
    with raw_primitive.new_raw_decrypting_stream(
        ct_source, aad, close_ciphertext_source=True) as ds:
      data = bytearray(b'xxxxx')
      n = ds.readinto(data)  # writes 5 bytes into data.
      self.assertEqual(n, 5)
      self.assertEqual(data, b'plain')
      n = ds.readinto(data)  # writes remaining 4 bytes, leave the rest
      self.assertEqual(n, 4)
      self.assertEqual(data, b'textn')

  def test_raw_encrypt_decrypt_empty(self):
    raw_primitive = new_raw_primitive()
    plaintext = b''
    aad = b''
    ct_destination = bytes_io.BytesIOWithValueAfterClose()
    with raw_primitive.new_raw_encrypting_stream(ct_destination, aad) as es:
      es.write(plaintext)

    ct_source = io.BytesIO(ct_destination.value_after_close())
    with raw_primitive.new_raw_decrypting_stream(
        ct_source, aad, close_ciphertext_source=True) as ds:
      self.assertEqual(ds.read(5), b'')

  def test_raw_read_after_eof_returns_empty_bytes(self):
    raw_primitive = new_raw_primitive()
    plaintext = b'plaintext' + B_X80
    aad = b'associated_data' + B_X80

    ct_destination = bytes_io.BytesIOWithValueAfterClose()
    with raw_primitive.new_raw_encrypting_stream(ct_destination, aad) as es:
      self.assertLen(plaintext, es.write(plaintext))

    ct_source = io.BytesIO(ct_destination.value_after_close())
    with raw_primitive.new_raw_decrypting_stream(
        ct_source, aad, close_ciphertext_source=True) as ds:
      _ = ds.readall()
      self.assertEqual(ds.read(100), b'')

  def test_raw_encrypt_decrypt_close(self):
    raw_primitive = new_raw_primitive()
    plaintext = b'plaintext' + B_X80
    aad = b'associated_data' + B_X80

    # Encrypt
    ct_destination = bytes_io.BytesIOWithValueAfterClose()
    es = raw_primitive.new_raw_encrypting_stream(ct_destination, aad)
    es.write(plaintext)
    self.assertFalse(ct_destination.closed)
    self.assertFalse(es.closed)
    es.close()
    self.assertTrue(ct_destination.closed)
    self.assertTrue(es.closed)

    # Decrypt, with and without close_ciphertext_source
    for close_ciphertext_source in [True, False]:
      ct_source = io.BytesIO(ct_destination.value_after_close())
      ds = raw_primitive.new_raw_decrypting_stream(
          ct_source, aad,
          close_ciphertext_source=close_ciphertext_source)
      self.assertFalse(ct_source.closed)
      self.assertFalse(ds.closed)
      ds.close()
      self.assertEqual(ct_source.closed, close_ciphertext_source)
      self.assertTrue(ds.closed)

  def test_raw_encrypt_decrypt_wrong_aad(self):
    raw_primitive = new_raw_primitive()
    plaintext = b'plaintext' + B_X80
    aad = b'associated_data' + B_X80

    # Encrypt
    ct_destination = bytes_io.BytesIOWithValueAfterClose()
    with raw_primitive.new_raw_encrypting_stream(ct_destination, aad) as es:
      self.assertLen(plaintext, es.write(plaintext))
    self.assertNotEqual(ct_destination.value_after_close(), plaintext)

    # Decrypt
    ct_source = io.BytesIO(ct_destination.value_after_close())
    with raw_primitive.new_raw_decrypting_stream(
        ct_source, b'bad' + aad, close_ciphertext_source=True) as ds:
      with self.assertRaises(core.TinkError):
        ds.read()

  @parameterized.parameters([
      streaming_aead.streaming_aead_key_templates.AES128_GCM_HKDF_4KB,
      streaming_aead.streaming_aead_key_templates.AES128_GCM_HKDF_1MB,
      streaming_aead.streaming_aead_key_templates.AES256_GCM_HKDF_4KB,
      streaming_aead.streaming_aead_key_templates.AES256_GCM_HKDF_1MB,
      streaming_aead.streaming_aead_key_templates.AES128_CTR_HMAC_SHA256_4KB,
      streaming_aead.streaming_aead_key_templates.AES128_CTR_HMAC_SHA256_1MB,
      streaming_aead.streaming_aead_key_templates.AES256_CTR_HMAC_SHA256_4KB,
      streaming_aead.streaming_aead_key_templates.AES256_CTR_HMAC_SHA256_1MB
  ])
  def test_encrypt_decrypt_success(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)

    plaintext = b'plaintext'
    associated_data = b'associated_data'

    # Encrypt
    ciphertext_destination = bytes_io.BytesIOWithValueAfterClose()
    with primitive.new_encrypting_stream(ciphertext_destination,
                                         associated_data) as encryption_stream:
      encryption_stream.write(plaintext)

    ciphertext = ciphertext_destination.value_after_close()

    # Decrypt
    ciphertext_source = io.BytesIO(ciphertext)
    decrypted = None
    with primitive.new_decrypting_stream(ciphertext_source,
                                         associated_data) as decryption_stream:
      decrypted = decryption_stream.read()

    self.assertEqual(decrypted, plaintext)

if __name__ == '__main__':
  absltest.main()



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Monitoring for Tink.

Defines entities related to monitoring of key usage.
"""

import abc
from typing import Callable, Dict, Optional

from tink.proto import tink_pb2


Annotations = Dict[str, str]


class MonitoringKeySetInfo:
  """Immutable representation of a KeySet in a certain point in time for the purpose of monitoring operations involving cryptographic keys."""

  def __init__(
      self,
      annotations: Optional[Annotations],
      keyset_info: tink_pb2.KeysetInfo,
  ):
    self._annotations = annotations
    self._keyset_info = keyset_info

  def get_annotations(self) -> Optional[Annotations]:
    return self._annotations

  def get_keyset_info(self) -> tink_pb2.KeysetInfo:
    return self._keyset_info


class MonitoringContext:
  """Context for monitoring events, consisting of the primitive, API used and info on the keyset."""

  def __init__(
      self,
      primitive: str,
      api_function: str,
      keyset_info: MonitoringKeySetInfo,
  ):
    self._primitive = primitive
    self._api_function = api_function
    self._keyset_info = keyset_info

  def get_primitive(self) -> str:
    return self._primitive

  def get_api_function(self) -> str:
    return self._api_function

  def get_keyset_info(self) -> MonitoringKeySetInfo:
    return self._keyset_info


class KeyUsageMonitor(metaclass=abc.ABCMeta):
  """Interface for monitoring the usage of keys.

  This interface is used by all Tink primitives to report key usage.
  """

  @abc.abstractmethod
  def log(self, key_id: int, num_bytes_as_input: int):
    """Logs a successful use of `key_id` on an input of `num_bytes_as_input`.

    Tink primitive wrappers call this method when they successfully used a key
    to
    carry out a primitive method, e.g. Aead.encrypt(). As a consequence,
    subclasses should be mindful on the amount of work
    performed by this method, as this will be called on each cryptographic
    operation. Implementations are responsible to add
    context to identify, e.g., the primitive and the API function.

    Args:
      key_id: The key ID of the used key.
      num_bytes_as_input: The size of the input used for the cryptographic
        operation.
    """
    pass

  @abc.abstractmethod
  def log_failure(self):
    """Logs a failure of a cryptographic operation.

    Tink calls this method when a cryptographic operation
    fails, e.g. no key could be found to decrypt a ciphertext. In this
    case the failure is not associated with a specific key, therefore this
    method has no arguments. The MonitoringClient implementation is responsible
    to add context to identify where the failure comes from.
    """
    pass


_registered_key_usage_monitor_factory: Callable[
    [MonitoringContext], KeyUsageMonitor
] = None


def register_key_usage_monitor_factory(
    f: Callable[[MonitoringContext], KeyUsageMonitor],
):
  """Registers a factory for creating KeyUsageMonitor objects.

  The factory will be called in the `wrap` method of the primitive wrappers
  and passed to the respective primitives to be used when using the keys.

  Args:
    f: A factory function that returns a KeyUsageMonitor object. Called every
      time the monitor is requested with `get_monitor_or_none`.
  """
  global _registered_key_usage_monitor_factory
  _registered_key_usage_monitor_factory = f


def get_key_usage_monitor_or_none(
    monitoring_context: MonitoringContext,
) -> Optional[KeyUsageMonitor]:
  """Returns a new instance of a KeyUsageMonitor or None.

  The instance is created using the registered factory, passing keyset_info
  along. None is returned if there is no registered factory.

  Args:
    monitoring_context: Context object with all monitoring-related information.

  Returns:
    A new instance of a KeyUsageMonitor or None.
  """
  return (
      _registered_key_usage_monitor_factory(monitoring_context)
      if _registered_key_usage_monitor_factory is not None
      else None
  )



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import struct

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import aes_gcm_pb2
import tink
from tink import aead
from tink import core
from tink import mac
from tink.testing import fake_kms


def setUpModule():
  aead.register()


class KmsEnvelopeAeadTest(parameterized.TestCase):

  def remote_aead(self):
    keyset_handle = tink.new_keyset_handle(aead.aead_key_templates.AES256_GCM)
    return keyset_handle.primitive(aead.Aead)

  @parameterized.parameters([
      aead.aead_key_templates.AES128_EAX,
      aead.aead_key_templates.AES256_EAX,
      aead.aead_key_templates.AES128_GCM,
      aead.aead_key_templates.AES256_GCM,
      aead.aead_key_templates.AES128_GCM_SIV,
      aead.aead_key_templates.AES256_GCM_SIV,
      aead.aead_key_templates.AES128_CTR_HMAC_SHA256,
      aead.aead_key_templates.AES256_CTR_HMAC_SHA256,
      aead.aead_key_templates.XCHACHA20_POLY1305,
  ])
  def test_encrypt_decrypt(self, dek_template):
    env_aead = aead.KmsEnvelopeAead(dek_template, self.remote_aead())

    plaintext = b'plaintext'
    associated_data = b'associated_data'
    ciphertext = env_aead.encrypt(plaintext, associated_data)
    self.assertEqual(plaintext, env_aead.decrypt(ciphertext, associated_data))

    with self.assertRaises(core.TinkError):
      _ = env_aead.decrypt(ciphertext, b'invalid_associated_data')

  def test_encrypt_decrypt_missing_ad(self):
    key_template = aead.aead_key_templates.AES256_GCM
    env_aead = aead.KmsEnvelopeAead(key_template, self.remote_aead())

    plaintext = b'helloworld'
    ciphertext = env_aead.encrypt(plaintext, b'envelope_ad')
    with self.assertRaises(core.TinkError):
      _ = env_aead.decrypt(ciphertext, b'')

  def test_invalid_dek_template_fails(self):
    remote_aead = self.remote_aead()
    with self.assertRaises(tink.TinkError):
      aead.KmsEnvelopeAead(
          mac.mac_key_templates.HMAC_SHA256_128BITTAG, remote_aead
      )

  def test_corrupted_ciphertext(self):
    dek_template = aead.aead_key_templates.AES256_GCM
    env_aead = aead.KmsEnvelopeAead(dek_template, self.remote_aead())

    plaintext = b'helloworld'
    ciphertext = bytearray(env_aead.encrypt(plaintext, b'some ad'))
    ciphertext[-1] ^= 0x1
    corrupted_ciphertext = bytes(ciphertext)

    with self.assertRaises(core.TinkError):
      _ = env_aead.decrypt(corrupted_ciphertext, b'some ad')

  def test_corrupted_dek(self):
    dek_template = aead.aead_key_templates.AES256_GCM
    env_aead = aead.KmsEnvelopeAead(dek_template, self.remote_aead())

    plaintext = b'helloworld'
    ciphertext = bytearray(env_aead.encrypt(plaintext, b'some ad'))
    ciphertext[4] ^= 0x1
    corrupted_ciphertext = bytes(ciphertext)

    with self.assertRaises(core.TinkError):
      _ = env_aead.decrypt(corrupted_ciphertext, b'some ad')

  def test_ciphertext_too_short(self):
    dek_template = aead.aead_key_templates.AES256_GCM
    env_aead = aead.KmsEnvelopeAead(dek_template, self.remote_aead())

    with self.assertRaises(core.TinkError):
      _ = env_aead.decrypt(b'foo', b'some ad')

  def test_encrypted_dek_size_too_large(self):
    dek_template = aead.aead_key_templates.AES256_GCM
    env_aead = aead.KmsEnvelopeAead(dek_template, self.remote_aead())

    ciphertext = b'\x88\x88\x88\x88\x88\x88\x88\x88\x88\x88\x88\x88'
    with self.assertRaisesRegex(
        core.TinkError, 'length of encrypted DEK too large'
    ):
      _ = env_aead.decrypt(ciphertext, b'')

  def test_malformed_dek_length(self):
    dek_template = aead.aead_key_templates.AES256_GCM
    env_aead = aead.KmsEnvelopeAead(dek_template, self.remote_aead())

    plaintext = b'helloworld'
    ciphertext = bytearray(env_aead.encrypt(plaintext, b'some ad'))
    ciphertext[0:3] = [0xff, 0xff, 0xff, 0xff]
    corrupted_ciphertext = bytes(ciphertext)

    with self.assertRaises(core.TinkError):
      _ = env_aead.decrypt(corrupted_ciphertext, b'some ad')

    ciphertext[0:3] = [0, 0, 0, 0]
    corrupted_ciphertext = bytes(ciphertext)

    with self.assertRaises(core.TinkError):
      _ = env_aead.decrypt(corrupted_ciphertext, b'some ad')

  def test_ciphertext_wire_format(self):
    dek_template = aead.aead_key_templates.AES256_GCM
    remote_aead = self.remote_aead()
    env_aead = aead.KmsEnvelopeAead(dek_template, remote_aead)

    plaintext = b'helloworld'
    ciphertext = bytearray(env_aead.encrypt(plaintext, b'some ad'))

    # test that ciphertext has the wire format described here:
    # https://developers.google.com/tink/wire-format#envelope_encryption
    dek_len = struct.unpack('>I',
                            ciphertext[0:aead.KmsEnvelopeAead.DEK_LEN_BYTES])[0]
    encrypted_dek_bytes = bytes(ciphertext[
        aead.KmsEnvelopeAead.DEK_LEN_BYTES:aead.KmsEnvelopeAead.DEK_LEN_BYTES +
        dek_len])
    dek_bytes = remote_aead.decrypt(encrypted_dek_bytes, b'')

    # Try to deserialize key
    key = aes_gcm_pb2.AesGcmKey.FromString(dek_bytes)
    self.assertLen(key.key_value, 32)

  def test_compatible_with_kms_envelope_aead_key(self):
    kms_uri = (
        'fake-kms://CM2b3_MDElQKSAowdHlwZS5nb29nbGVhcGlzLmNvbS9nb29nbGUuY3J5'
        'cHRvLnRpbmsuQWVzR2NtS2V5EhIaEIK75t5L-adlUwVhWvRuWUwYARABGM2b3_MDIAE'
    )
    dek_template = aead.aead_key_templates.AES256_GCM

    # Register kmsClient, and use create_kms_envelope_aead_key_template,
    # tink.new_keyset_handle and keyset_handle.primitive to create an Aead.
    fake_kms.register_client(key_uri=kms_uri)
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri=kms_uri, dek_template=dek_template
    )
    keyset_handle = tink.new_keyset_handle(template)
    aead1 = keyset_handle.primitive(aead.Aead)

    # Get Aead from the kms_client, and directly create the envelope AEAD
    # without the registry.
    kms_client = fake_kms.FakeKmsClient()
    remote_aead = kms_client.get_aead(kms_uri)
    aead2 = aead.KmsEnvelopeAead(dek_template, remote_aead)

    plaintext = b'plaintext'
    associated_data = b'associated_data'
    ciphertext1 = aead1.encrypt(plaintext, associated_data)
    self.assertEqual(aead2.decrypt(ciphertext1, associated_data), plaintext)

    ciphertext2 = aead2.encrypt(plaintext, associated_data)
    self.assertEqual(aead1.decrypt(ciphertext2, associated_data), plaintext)


if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for big_integer_util."""

from absl.testing import absltest
from absl.testing import parameterized

from tink.internal import big_integer_util


class BigIntegerUtilTest(parameterized.TestCase):

  def test_bytes_to_num(self):
    for i in range(100000):
      big_int_bytes = big_integer_util.num_to_bytes(i)
      self.assertEqual(int.from_bytes(big_int_bytes, byteorder='big'), i)

  @parameterized.named_parameters(
      ('0', 0, b'\x00'), ('255', 255, b'\xff'), ('256', 256, b'\x01\x00'),
      ('65535', 65535, b'\xff\xff'), ('65536', 65536, b'\x01\x00\x00'),
      ('65537', 65537, b'\x01\x00\x01'), ('65538', 65538, b'\x01\x00\x02'),
      ('16909060', 16909060, b'\x01\x02\x03\x04'))
  def test_num_to_bytes(self, number, expected):
    self.assertEqual(big_integer_util.num_to_bytes(number), expected)

  def test_num_to_bytes_minus_one_overflow(self):
    with self.assertRaises(OverflowError):
      big_integer_util.num_to_bytes(-1)


if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""FileObjectAdapter class.

Used in conjunction with PythonOutputStream/PythonInputStream to allow a C++
OutputStream/InputStream to interact with a Python file-like object.
"""

import io
from typing import BinaryIO

from tink.cc.pybind import tink_bindings


class FileObjectAdapter(tink_bindings.PythonFileObjectAdapter):
  """Adapts a Python file object for use in C++."""

  def __init__(self, file_object: BinaryIO):
    # Required to fix CLIF "Value invalidated due to capture by std::unique_ptr"
    super().__init__()
    self._file_object = file_object

  def write(self, data: bytes) -> int:
    """Writes to underlying file object and returns number of bytes written."""
    try:
      written = self._file_object.write(data)
      if written is None:
        return 0
      if written < 0 or written > len(data):
        raise ValueError('invalid value returned by write')
      return written
    except io.BlockingIOError as e:
      if e.characters_written is None:
        return 0
      if e.characters_written < 0 or e.characters_written > len(data):
        raise ValueError('invalid characters_written') from e
      return e.characters_written

  def close(self) -> None:
    self._file_object.close()

  def read(self, size: int) -> bytes:
    """Reads at most 'size' bytes from the underlying file object.

    Args:
      size: A non-negative integer, maximum number of bytes to read.

    Returns:
      Bytes that were read. An empty bytes object is returned if no bytes are
      available at the moment.

    Raises:
      EOFError if the file object is already at EOF.
    """
    if size < 0:
      raise ValueError('size must be non-negative')

    try:
      data = self._file_object.read(size)
      if data is None:
        return b''
      elif not data and size > 0:
        raise EOFError('EOF')
      return data
    except io.BlockingIOError:
      return b''



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/common.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/common.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17tink/proto/common.proto\x12\x12google.crypto.tink*c\n\x11\x45llipticCurveType\x12\x11\n\rUNKNOWN_CURVE\x10\x00\x12\r\n\tNIST_P256\x10\x02\x12\r\n\tNIST_P384\x10\x03\x12\r\n\tNIST_P521\x10\x04\x12\x0e\n\nCURVE25519\x10\x05*j\n\rEcPointFormat\x12\x12\n\x0eUNKNOWN_FORMAT\x10\x00\x12\x10\n\x0cUNCOMPRESSED\x10\x01\x12\x0e\n\nCOMPRESSED\x10\x02\x12#\n\x1f\x44O_NOT_USE_CRUNCHY_UNCOMPRESSED\x10\x03*V\n\x08HashType\x12\x10\n\x0cUNKNOWN_HASH\x10\x00\x12\x08\n\x04SHA1\x10\x01\x12\n\n\x06SHA384\x10\x02\x12\n\n\x06SHA256\x10\x03\x12\n\n\x06SHA512\x10\x04\x12\n\n\x06SHA224\x10\x05\x42Y\n\x1c\x63om.google.crypto.tink.protoP\x01Z7github.com/tink-crypto/tink-go/v2/proto/common_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z7github.com/tink-crypto/tink-go/v2/proto/common_go_proto'
  _globals['_ELLIPTICCURVETYPE']._serialized_start=47
  _globals['_ELLIPTICCURVETYPE']._serialized_end=146
  _globals['_ECPOINTFORMAT']._serialized_start=148
  _globals['_ECPOINTFORMAT']._serialized_end=254
  _globals['_HASHTYPE']._serialized_start=256
  _globals['_HASHTYPE']._serialized_end=342
# @@protoc_insertion_point(module_scope)



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink._mac_key_templates."""

from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import common_pb2
from tink.proto import hmac_pb2
from tink.proto import tink_pb2
from tink import mac


class MacKeyTemplatesTest(parameterized.TestCase):

  def test_create_hmac_key_template(self):
    # Intentionally using "weird" or invalid values for parameters,
    # to test that the function correctly puts them in the resulting template.
    template = None
    with self.assertWarns(DeprecationWarning):
      template = mac.mac_key_templates.create_hmac_key_template(
          key_size=42, tag_size=24, hash_type=common_pb2.SHA512)
    self.assertEqual('type.googleapis.com/google.crypto.tink.HmacKey',
                     template.type_url)
    self.assertEqual(tink_pb2.TINK, template.output_prefix_type)
    key_format = hmac_pb2.HmacKeyFormat.FromString(template.value)
    self.assertEqual(42, key_format.key_size)
    self.assertEqual(24, key_format.params.tag_size)
    self.assertEqual(common_pb2.SHA512, key_format.params.hash)

  def test_create_aes_cmac_key_template(self):
    template = None
    with self.assertWarns(DeprecationWarning):
      template = mac.mac_key_templates.create_aes_cmac_key_template(
          key_size=42, tag_size=24)
    self.assertEqual('type.googleapis.com/google.crypto.tink.AesCmacKey',
                     template.type_url)

if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
"""Tests for tink.python.tink.jwt._jwt_validator."""

import datetime

from absl.testing import absltest
from tink import jwt
from tink.jwt import _jwt_validator


class JwtValidatorTest(absltest.TestCase):

  def test_validator_getters(self):
    fixed_now = datetime.datetime.fromtimestamp(12345, datetime.timezone.utc)
    clock_skew = datetime.timedelta(minutes=1)
    validator = jwt.new_validator(
        expected_type_header='type_header',
        expected_issuer='issuer',
        expected_audience='audience',
        fixed_now=fixed_now,
        clock_skew=clock_skew)
    self.assertTrue(validator.has_expected_type_header())
    self.assertEqual(validator.expected_type_header(), 'type_header')
    self.assertTrue(validator.has_expected_issuer())
    self.assertEqual(validator.expected_issuer(), 'issuer')
    self.assertTrue(validator.has_expected_audience())
    self.assertEqual(validator.expected_audience(), 'audience')
    self.assertFalse(validator.allow_missing_expiration())
    self.assertFalse(validator.ignore_issuer())
    self.assertFalse(validator.ignore_audiences())
    self.assertTrue(validator.has_fixed_now())
    self.assertEqual(validator.fixed_now(), fixed_now)
    self.assertEqual(validator.clock_skew(), clock_skew)

  def test_validator_ignore_getters(self):
    validator = jwt.new_validator(
        allow_missing_expiration=True,
        ignore_type_header=True,
        ignore_issuer=True,
        ignore_audiences=True)
    self.assertTrue(validator.allow_missing_expiration())
    self.assertTrue(validator.ignore_type_header())
    self.assertTrue(validator.ignore_issuer())
    self.assertTrue(validator.ignore_audiences())

  def test_empty_validator_getters(self):
    validator = jwt.new_validator()
    self.assertFalse(validator.has_expected_type_header())
    self.assertFalse(validator.has_expected_issuer())
    self.assertFalse(validator.has_expected_audience())
    self.assertFalse(validator.has_fixed_now())
    self.assertFalse(validator.clock_skew(), datetime.timedelta())

  def test_too_much_clock_skew(self):
    with self.assertRaises(ValueError):
      jwt.new_validator(clock_skew=datetime.timedelta(minutes=20))

  def test_validate_expired_fails(self):
    expired = (datetime.datetime.now(tz=datetime.timezone.utc)
               - datetime.timedelta(minutes=1))
    token = jwt.new_raw_jwt(expiration=expired)
    validator = jwt.new_validator()
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_validate_not_expired_success(self):
    still_valid = (datetime.datetime.now(tz=datetime.timezone.utc)
                   + datetime.timedelta(minutes=1))
    token = jwt.new_raw_jwt(expiration=still_valid)
    validator = jwt.new_validator()
    _jwt_validator.validate(validator, token)

  def test_validate_token_that_expires_now_fails(self):
    now = datetime.datetime.fromtimestamp(1234.0, tz=datetime.timezone.utc)
    token = jwt.new_raw_jwt(expiration=now)
    validator = jwt.new_validator()
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_validate_recently_expired_with_clock_skew_success(self):
    recently_expired = (datetime.datetime.now(tz=datetime.timezone.utc)
                        - datetime.timedelta(minutes=1))
    token = jwt.new_raw_jwt(expiration=recently_expired)
    validator = jwt.new_validator(clock_skew=datetime.timedelta(minutes=2))
    # because of clock_skew, the recently expired token is valid
    _jwt_validator.validate(validator, token)

  def test_validate_not_before_in_the_future_fails(self):
    in_the_future = (datetime.datetime.now(tz=datetime.timezone.utc)
                     + datetime.timedelta(minutes=1))
    token = jwt.new_raw_jwt(not_before=in_the_future, without_expiration=True)
    validator = jwt.new_validator(allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_validate_not_before_in_the_past_success(self):
    in_the_past = (datetime.datetime.now(tz=datetime.timezone.utc)
                   - datetime.timedelta(minutes=1))
    token = jwt.new_raw_jwt(not_before=in_the_past, without_expiration=True)
    validator = jwt.new_validator(allow_missing_expiration=True)
    _jwt_validator.validate(validator, token)

  def test_validate_not_before_is_now_success(self):
    now = datetime.datetime.fromtimestamp(12345, datetime.timezone.utc)
    token = jwt.new_raw_jwt(not_before=now, without_expiration=True)
    validator = jwt.new_validator(allow_missing_expiration=True)
    _jwt_validator.validate(validator, token)

  def test_validate_not_before_almost_reached_with_clock_skew_success(self):
    in_one_minute = (datetime.datetime.now(tz=datetime.timezone.utc)
                     + datetime.timedelta(minutes=1))
    token = jwt.new_raw_jwt(not_before=in_one_minute, without_expiration=True)
    validator = jwt.new_validator(
        allow_missing_expiration=True, clock_skew=datetime.timedelta(minutes=2))
    _jwt_validator.validate(validator, token)

  def test_validate_issued_at(self):
    in_one_minute = (datetime.datetime.now(tz=datetime.timezone.utc)
                     + datetime.timedelta(minutes=1))
    one_minute_ago = (datetime.datetime.now(tz=datetime.timezone.utc)
                      - datetime.timedelta(minutes=1))
    token_with_issued_at_in_the_future = jwt.new_raw_jwt(
        issued_at=in_one_minute, without_expiration=True)
    token_with_issued_at_in_the_past = jwt.new_raw_jwt(
        issued_at=one_minute_ago, without_expiration=True)
    token_without_issued_at = jwt.new_raw_jwt(without_expiration=True)

    validator = jwt.new_validator(allow_missing_expiration=True)
    _jwt_validator.validate(validator, token_with_issued_at_in_the_future)
    _jwt_validator.validate(validator, token_with_issued_at_in_the_past)
    _jwt_validator.validate(validator, token_without_issued_at)

    issued_at_validator = jwt.new_validator(
        expect_issued_in_the_past=True, allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(issued_at_validator,
                              token_with_issued_at_in_the_future)
    _jwt_validator.validate(issued_at_validator,
                            token_with_issued_at_in_the_past)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(issued_at_validator, token_without_issued_at)

  def test_validate_issued_at_with_clock_skew(self):
    in_one_minute = (datetime.datetime.now(tz=datetime.timezone.utc)
                     + datetime.timedelta(minutes=1))
    token_one_minute_in_the_future = jwt.new_raw_jwt(
        issued_at=in_one_minute, without_expiration=True)

    validator_without_clock_skew = jwt.new_validator(
        expect_issued_in_the_past=True, allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator_without_clock_skew,
                              token_one_minute_in_the_future)

    validator_with_clock_skew = jwt.new_validator(
        expect_issued_in_the_past=True,
        allow_missing_expiration=True,
        clock_skew=datetime.timedelta(minutes=2))
    _jwt_validator.validate(validator_with_clock_skew,
                            token_one_minute_in_the_future)

  def test_requires_type_header_but_no_type_header_set_fails(self):
    token = jwt.new_raw_jwt(without_expiration=True)
    validator = jwt.new_validator(
        expected_type_header='type_header', allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_invalid_type_header_fails(self):
    token = jwt.new_raw_jwt(type_header='unknown', without_expiration=True)
    validator = jwt.new_validator(
        expected_type_header='type_header', allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_correct_type_header_success(self):
    token = jwt.new_raw_jwt(type_header='type_header', without_expiration=True)
    validator = jwt.new_validator(
        expected_type_header='type_header', allow_missing_expiration=True)
    _jwt_validator.validate(validator, token)

  def test_type_header_in_token_but_not_in_validator_fails(self):
    validator = jwt.new_validator(allow_missing_expiration=True)
    token_with_type_header = jwt.new_raw_jwt(
        type_header='type_header', without_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token_with_type_header)

  def test_ignore_type_header_success(self):
    validator = jwt.new_validator(
        ignore_type_header=True, allow_missing_expiration=True)
    token_without_type_header = jwt.new_raw_jwt(without_expiration=True)
    _jwt_validator.validate(validator, token_without_type_header)
    token_with_type_header = jwt.new_raw_jwt(
        type_header='type_header', without_expiration=True)
    _jwt_validator.validate(validator, token_with_type_header)

  def test_requires_issuer_but_no_issuer_set_fails(self):
    token = jwt.new_raw_jwt(without_expiration=True)
    validator = jwt.new_validator(
        expected_issuer='issuer', allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_invalid_issuer_fails(self):
    token = jwt.new_raw_jwt(issuer='unknown', without_expiration=True)
    validator = jwt.new_validator(
        expected_issuer='issuer', allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_correct_issuer_success(self):
    token = jwt.new_raw_jwt(issuer='issuer', without_expiration=True)
    validator = jwt.new_validator(
        expected_issuer='issuer', allow_missing_expiration=True)
    _jwt_validator.validate(validator, token)

  def test_issuer_in_token_but_not_in_validator_fails(self):
    validator = jwt.new_validator(allow_missing_expiration=True)
    token_with_issuer = jwt.new_raw_jwt(
        issuer='issuer', without_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token_with_issuer)

  def test_ignore_issuer_success(self):
    validator = jwt.new_validator(
        ignore_issuer=True, allow_missing_expiration=True)
    token_without_issuer = jwt.new_raw_jwt(without_expiration=True)
    _jwt_validator.validate(validator, token_without_issuer)
    token_with_issuer = jwt.new_raw_jwt(
        issuer='issuer', without_expiration=True)
    _jwt_validator.validate(validator, token_with_issuer)

  def test_requires_audience_but_no_audience_set_fails(self):
    token = jwt.new_raw_jwt(without_expiration=True)
    validator = jwt.new_validator(
        expected_audience='audience', allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_wrong_audience_fails(self):
    token = jwt.new_raw_jwt(audiences=['unknown'], without_expiration=True)
    validator = jwt.new_validator(
        expected_audience='audience', allow_missing_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_correct_audience_success(self):
    token = jwt.new_raw_jwt(audiences=['you', 'me'], without_expiration=True)
    validator = jwt.new_validator(
        expected_audience='me', allow_missing_expiration=True)
    _jwt_validator.validate(validator, token)

  def test_audience_in_token_but_not_in_validator_fails(self):
    validator = jwt.new_validator(allow_missing_expiration=True)
    token_with_audience = jwt.new_raw_jwt(
        audiences=['audience'], without_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token_with_audience)

  def test_no_audience_success(self):
    validator = jwt.new_validator(allow_missing_expiration=True)
    token = jwt.new_raw_jwt(without_expiration=True)
    _jwt_validator.validate(validator, token)

  def test_ignore_audiences_success(self):
    validator = jwt.new_validator(
        ignore_audiences=True, allow_missing_expiration=True)
    token_without_audience = jwt.new_raw_jwt(without_expiration=True)
    _jwt_validator.validate(validator, token_without_audience)
    token_with_audience = jwt.new_raw_jwt(
        audiences=['audience'], without_expiration=True)
    _jwt_validator.validate(validator, token_with_audience)

  def test_validate_with_fixed_now_expired_fails(self):
    in_two_minutes = (
        datetime.datetime.now(tz=datetime.timezone.utc) +
        datetime.timedelta(minutes=2))
    in_one_minute = in_two_minutes - datetime.timedelta(minutes=1)
    token = jwt.new_raw_jwt(expiration=in_one_minute)
    validator = jwt.new_validator(fixed_now=in_two_minutes)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_validate_with_fixed_now_not_yet_valid_fails(self):
    two_minutes_ago = (
        datetime.datetime.now(tz=datetime.timezone.utc) -
        datetime.timedelta(minutes=2))
    one_minute_ago = two_minutes_ago + datetime.timedelta(minutes=1)
    token = jwt.new_raw_jwt(not_before=one_minute_ago, without_expiration=True)
    validator = jwt.new_validator(fixed_now=two_minutes_ago)
    with self.assertRaises(jwt.JwtInvalidError):
      _jwt_validator.validate(validator, token)

  def test_validate_with_fixed_now_valid_success(self):
    fixed_now = datetime.datetime.fromtimestamp(12345, datetime.timezone.utc)
    validator = jwt.new_validator(fixed_now=fixed_now)
    expiration = fixed_now + datetime.timedelta(minutes=1)
    not_before = fixed_now - datetime.timedelta(minutes=1)
    token = jwt.new_raw_jwt(expiration=expiration, not_before=not_before)
    _jwt_validator.validate(validator, token)

  def test_invalid_clock_skew_fail(self):
    with self.assertRaises(ValueError):
      jwt.new_validator(clock_skew=datetime.timedelta(minutes=1000))

  def test_fixed_now_without_timezone_fail(self):
    with self.assertRaises(ValueError):
      jwt.new_validator(fixed_now=datetime.datetime.fromtimestamp(12345))


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink._keyset_reader."""

from typing import cast
from absl.testing import absltest

from tink.proto import tink_pb2
import tink
from tink import core


class JsonKeysetReaderTest(absltest.TestCase):

  def test_read(self):
    json_keyset = """
        {
          "primaryKeyId": 42,
          "key": [
            {
              "keyData": {
                "typeUrl": "type.googleapis.com/google.crypto.tink.AesGcmKey",
                "keyMaterialType": "SYMMETRIC",
                "value": "GhCS/1+ejWpx68NfGt6ziYHd"
              },
              "outputPrefixType": "TINK",
              "keyId": 42,
              "status": "ENABLED"
            }
          ]
        }"""
    reader = tink.JsonKeysetReader(json_keyset)
    keyset = reader.read()
    self.assertEqual(keyset.primary_key_id, 42)
    self.assertLen(keyset.key, 1)

  def test_read_invalid(self):
    reader = tink.JsonKeysetReader('not json')
    with self.assertRaises(core.TinkError):
      reader.read()

  def test_read_rejects_negative_key_id(self):
    json_keyset = """
        {
          "primaryKeyId": -42,
          "key": [
            {
              "keyData": {
                "typeUrl": "type.googleapis.com/google.crypto.tink.AesGcmKey",
                "keyMaterialType": "SYMMETRIC",
                "value": "GhCS/1+ejWpx68NfGt6ziYHd"
              },
              "outputPrefixType": "TINK",
              "keyId": -42,
              "status": "ENABLED"
            }
          ]
        }"""
    reader = tink.JsonKeysetReader(json_keyset)
    with self.assertRaises(core.TinkError):
      reader.read()

  def test_read_rejects_key_id_larger_than_uint32(self):
    # 4294967296 = 2^32, which is too large for uint32.
    json_keyset = """
        {
          "primaryKeyId": 4294967296,
          "key": [
            {
              "keyData": {
                "typeUrl": "type.googleapis.com/google.crypto.tink.AesGcmKey",
                "keyMaterialType": "SYMMETRIC",
                "value": "GhCS/1+ejWpx68NfGt6ziYHd"
              },
              "outputPrefixType": "TINK",
              "keyId": 4294967296,
              "status": "ENABLED"
            }
          ]
        }"""
    reader = tink.JsonKeysetReader(json_keyset)
    with self.assertRaises(core.TinkError):
      reader.read()

  def test_read_encrypted(self):
    # encryptedKeyset is a base64-encoding of 'some ciphertext with keyset'
    json_encrypted_keyset = """
        {
          "encryptedKeyset": "c29tZSBjaXBoZXJ0ZXh0IHdpdGgga2V5c2V0",
          "keysetInfo": {
            "primaryKeyId": 42,
            "keyInfo": [
              {
                "typeUrl": "type.googleapis.com/google.crypto.tink.AesGcmKey",
                "outputPrefixType": "TINK",
                "keyId": 42,
                "status": "ENABLED"
              }
            ]
          }
        }"""
    reader = tink.JsonKeysetReader(json_encrypted_keyset)
    enc_keyset = reader.read_encrypted()
    self.assertEqual(enc_keyset.encrypted_keyset,
                     b'some ciphertext with keyset')
    self.assertLen(enc_keyset.keyset_info.key_info, 1)
    self.assertEqual(enc_keyset.keyset_info.key_info[0].type_url,
                     'type.googleapis.com/google.crypto.tink.AesGcmKey')

  def test_read_encrypted_invalid(self):
    reader = tink.JsonKeysetReader('not json')
    with self.assertRaises(core.TinkError):
      reader.read_encrypted()


class BinaryKeysetReaderTest(absltest.TestCase):

  def test_read(self):
    keyset = tink_pb2.Keyset()
    keyset.primary_key_id = 42
    key = keyset.key.add()
    key.key_data.type_url = 'type.googleapis.com/google.crypto.tink.AesGcmKey'
    key.key_data.key_material_type = tink_pb2.KeyData.SYMMETRIC
    key.key_data.value = b'GhCS/1+ejWpx68NfGt6ziYHd'
    key.output_prefix_type = tink_pb2.TINK
    key.key_id = 42
    key.status = tink_pb2.ENABLED
    reader = tink.BinaryKeysetReader(keyset.SerializeToString())
    self.assertEqual(keyset, reader.read())

  def test_read_none(self):
    with self.assertRaises(core.TinkError):
      reader = tink.BinaryKeysetReader(cast(bytes, None))
      reader.read()

  def test_read_empty(self):
    with self.assertRaises(core.TinkError):
      reader = tink.BinaryKeysetReader(b'')
      reader.read()

  def test_read_invalid(self):
    with self.assertRaises(core.TinkError):
      reader = tink.BinaryKeysetReader(b'some weird data')
      reader.read()

  def test_read_encrypted(self):
    encrypted_keyset = tink_pb2.EncryptedKeyset()
    encrypted_keyset.encrypted_keyset = b'c29tZSBjaXBoZXJ0ZXh0IHdpdGgga2V5c2V0'
    encrypted_keyset.keyset_info.primary_key_id = 42
    key_info = encrypted_keyset.keyset_info.key_info.add()
    key_info.type_url = 'type.googleapis.com/google.crypto.tink.AesGcmKey'
    key_info.output_prefix_type = tink_pb2.TINK
    key_info.key_id = 42
    key_info.status = tink_pb2.ENABLED
    reader = tink.BinaryKeysetReader(
        encrypted_keyset.SerializeToString())
    self.assertEqual(encrypted_keyset, reader.read_encrypted())

  def test_read_encrypted_none(self):
    with self.assertRaises(core.TinkError):
      reader = tink.BinaryKeysetReader(cast(bytes, None))
      reader.read_encrypted()

  def test_read_encrypted_empty(self):
    with self.assertRaises(core.TinkError):
      reader = tink.BinaryKeysetReader(b'')
      reader.read_encrypted()

  def test_read_encrypted_invalid(self):
    with self.assertRaises(core.TinkError):
      reader = tink.BinaryKeysetReader(b'some weird data')
      reader.read_encrypted()


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.aead_wrapper."""

from absl.testing import absltest
from absl.testing import parameterized

import tink
from tink import _keyset_handle
from tink import _monitoring
from tink import aead
from tink import core
from tink.testing import fake_key_usage_monitor
from tink.testing import keyset_builder


AEAD_TEMPLATE = aead.aead_key_templates.AES128_EAX
RAW_AEAD_TEMPLATE = keyset_builder.raw_template(AEAD_TEMPLATE)


def setUpModule():
  aead.register()


class AeadWrapperTest(parameterized.TestCase):

  @parameterized.parameters([AEAD_TEMPLATE, RAW_AEAD_TEMPLATE])
  def test_encrypt_decrypt(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)
    ciphertext = primitive.encrypt(b'plaintext', b'associated_data')
    self.assertEqual(
        primitive.decrypt(ciphertext, b'associated_data'), b'plaintext'
    )

  @parameterized.parameters([AEAD_TEMPLATE, RAW_AEAD_TEMPLATE])
  def test_decrypt_unknown_ciphertext_fails(self, template):
    unknown_handle = tink.new_keyset_handle(template)
    unknown_primitive = unknown_handle.primitive(aead.Aead)
    unknown_ciphertext = unknown_primitive.encrypt(
        b'plaintext', b'associated_data'
    )

    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)

    with self.assertRaises(tink.TinkError):
      primitive.decrypt(unknown_ciphertext, b'associated_data')

  @parameterized.parameters([AEAD_TEMPLATE, RAW_AEAD_TEMPLATE])
  def test_decrypt_wrong_associated_data_fails(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)

    ciphertext = primitive.encrypt(b'plaintext', b'associated_data')
    with self.assertRaises(tink.TinkError):
      primitive.decrypt(ciphertext, b'wrong_associated_data')

  @parameterized.parameters([
      (AEAD_TEMPLATE, AEAD_TEMPLATE),
      (RAW_AEAD_TEMPLATE, AEAD_TEMPLATE),
      (AEAD_TEMPLATE, RAW_AEAD_TEMPLATE),
      (RAW_AEAD_TEMPLATE, RAW_AEAD_TEMPLATE),
  ])
  def test_encrypt_decrypt_with_key_rotation(self, template1, template2):
    builder = keyset_builder.new_keyset_builder()
    older_key_id = builder.add_new_key(template1)
    builder.set_primary_key(older_key_id)
    p1 = builder.keyset_handle().primitive(aead.Aead)

    newer_key_id = builder.add_new_key(template2)
    p2 = builder.keyset_handle().primitive(aead.Aead)

    builder.set_primary_key(newer_key_id)
    p3 = builder.keyset_handle().primitive(aead.Aead)

    builder.disable_key(older_key_id)
    p4 = builder.keyset_handle().primitive(aead.Aead)

    self.assertNotEqual(older_key_id, newer_key_id)
    # p1 encrypts with the older key. So p1, p2 and p3 can decrypt it,
    # but not p4.
    ciphertext1 = p1.encrypt(b'plaintext', b'ad')
    self.assertEqual(p1.decrypt(ciphertext1, b'ad'), b'plaintext')
    self.assertEqual(p2.decrypt(ciphertext1, b'ad'), b'plaintext')
    self.assertEqual(p3.decrypt(ciphertext1, b'ad'), b'plaintext')
    with self.assertRaises(tink.TinkError):
      _ = p4.decrypt(ciphertext1, b'ad')

    # p2 encrypts with the older key. So p1, p2 and p3 can decrypt it,
    # but not p4.
    ciphertext2 = p2.encrypt(b'plaintext', b'ad')
    self.assertEqual(p1.decrypt(ciphertext2, b'ad'), b'plaintext')
    self.assertEqual(p2.decrypt(ciphertext2, b'ad'), b'plaintext')
    self.assertEqual(p3.decrypt(ciphertext2, b'ad'), b'plaintext')
    with self.assertRaises(tink.TinkError):
      _ = p4.decrypt(ciphertext2, b'ad')

    # p3 encrypts with the newer key. So p2, p3 and p4 can decrypt it,
    # but not p1.
    ciphertext3 = p3.encrypt(b'plaintext', b'ad')
    with self.assertRaises(tink.TinkError):
      _ = p1.decrypt(ciphertext3, b'ad')
    self.assertEqual(p2.decrypt(ciphertext3, b'ad'), b'plaintext')
    self.assertEqual(p3.decrypt(ciphertext3, b'ad'), b'plaintext')
    self.assertEqual(p4.decrypt(ciphertext3, b'ad'), b'plaintext')

    # p4 encrypts with the newer key. So p2, p3 and p4 can decrypt it,
    # but not p1.
    ciphertext4 = p4.encrypt(b'plaintext', b'ad')
    with self.assertRaises(tink.TinkError):
      _ = p1.decrypt(ciphertext4, b'ad')
    self.assertEqual(p2.decrypt(ciphertext4, b'ad'), b'plaintext')
    self.assertEqual(p3.decrypt(ciphertext4, b'ad'), b'plaintext')
    self.assertEqual(p4.decrypt(ciphertext4, b'ad'), b'plaintext')


class KeyUsageMonitorTest(absltest.TestCase):

  def setUp(self):
    super().setUp()
    self.encryption_key_usage_monitor = (
        fake_key_usage_monitor.FakeKeyUsageMonitor()
    )
    self.decryption_key_usage_monitor = (
        fake_key_usage_monitor.FakeKeyUsageMonitor()
    )
    _monitoring.register_key_usage_monitor_factory(
        lambda context: (
            self.encryption_key_usage_monitor
            if context.get_api_function() == 'encrypt'
            else self.decryption_key_usage_monitor
        )
    )

  def test_key_usage_monitor_log(self):
    keyset_handle = _keyset_handle._new_keyset_handle_with_annotations(
        AEAD_TEMPLATE, {'test': 'test'}
    )
    primitive = keyset_handle.primitive(aead.Aead)

    ciphertext = primitive.encrypt(b'plaintext', b'associated_data')
    primitive.decrypt(ciphertext, b'associated_data')

    self.assertSequenceEqual(
        self.encryption_key_usage_monitor.log_calls,
        [(keyset_handle.keyset_info().key_info[0].key_id, len(b'plaintext'))],
    )
    self.assertSequenceEqual(
        self.decryption_key_usage_monitor.log_calls,
        [(
            keyset_handle.keyset_info().key_info[0].key_id,
            len(ciphertext) - core.crypto_format.NON_RAW_PREFIX_SIZE,
        )],
    )

  def test_key_usage_monitor_log_failure(self):
    keyset_handle = _keyset_handle._new_keyset_handle_with_annotations(
        AEAD_TEMPLATE, {'test': 'test'}
    )
    primitive = keyset_handle.primitive(aead.Aead)

    try:
      primitive.decrypt(b'x', b'context')
    except core.TinkError:
      pass

    self.assertEqual(
        self.decryption_key_usage_monitor.log_failure_calls_count, 1
    )


if __name__ == '__main__':
  absltest.main()



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Provides a fake KeyUsageMonitor for testing."""

from typing import List, Tuple

from tink import _monitoring


class FakeKeyUsageMonitor(_monitoring.KeyUsageMonitor):
  """Implementation of KeyUsageMonitor that only records its method calls."""

  def __init__(self):
    self.log_calls: List[Tuple[int, int]] = []
    self.log_failure_calls_count: int = 0

  def log(self, key_id: int, num_bytes_as_input: int) -> None:
    self.log_calls.append((key_id, num_bytes_as_input))

  def log_failure(self) -> None:
    self.log_failure_calls_count += 1



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/aes_eax.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/aes_eax.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18tink/proto/aes_eax.proto\x12\x12google.crypto.tink\"\x1f\n\x0c\x41\x65sEaxParams\x12\x0f\n\x07iv_size\x18\x01 \x01(\r\"U\n\x0f\x41\x65sEaxKeyFormat\x12\x30\n\x06params\x18\x01 \x01(\x0b\x32 .google.crypto.tink.AesEaxParams\x12\x10\n\x08key_size\x18\x02 \x01(\r\"a\n\tAesEaxKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x30\n\x06params\x18\x02 \x01(\x0b\x32 .google.crypto.tink.AesEaxParams\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x42Z\n\x1c\x63om.google.crypto.tink.protoP\x01Z8github.com/tink-crypto/tink-go/v2/proto/aes_eax_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.aes_eax_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z8github.com/tink-crypto/tink-go/v2/proto/aes_eax_go_proto'
  _globals['_AESEAXPARAMS']._serialized_start=48
  _globals['_AESEAXPARAMS']._serialized_end=79
  _globals['_AESEAXKEYFORMAT']._serialized_start=81
  _globals['_AESEAXKEYFORMAT']._serialized_end=166
  _globals['_AESEAXKEY']._serialized_start=168
  _globals['_AESEAXKEY']._serialized_end=265
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A client for Google Cloud KMS."""

import re
from typing import Optional

from google.api_core import exceptions as core_exceptions
from google.auth import credentials as auth_credentials
from google.cloud import kms_v1
from google.oauth2 import service_account

import tink
from tink import aead

GCP_KEYURI_PREFIX = 'gcp-kms://'
_KMS_KEY_REGEX = re.compile(
    'projects/([^/]+)/'
    'locations/([a-zA-Z0-9_-]{1,63})/'
    'keyRings/([a-zA-Z0-9_-]{1,63})/'
    'cryptoKeys/([a-zA-Z0-9_-]{1,63})$'
)
_KMS_KEY_VERSION_REGEX = re.compile(
    'projects/([^/]+)/'
    'locations/([a-zA-Z0-9_-]{1,63})/'
    'keyRings/([a-zA-Z0-9_-]{1,63})/'
    'cryptoKeys/([a-zA-Z0-9_-]{1,63})/'
    'cryptoKeyVersions/([a-zA-Z0-9_-]{1,63})$'
)


class _GcpKmsAead(aead.Aead):
  """Implements the Aead interface for GCP KMS."""

  def __init__(
      self, client: kms_v1.KeyManagementServiceClient, key_name: str
  ) -> None:
    if not key_name:
      raise tink.TinkError('key_name cannot be null.')
    if not (
        _KMS_KEY_REGEX.match(key_name) or _KMS_KEY_VERSION_REGEX.match(key_name)
    ):
      raise tink.TinkError(
          'Invalid key_name format: {}.\nKMS keys should follow the format: '
          '"projects/<project-id>/locations/<location>/keyRings/<keyring>/'
          'cryptoKeys/<key-name>"'.format(key_name)
      )
    if not client:
      raise tink.TinkError('client cannot be null.')
    self.client = client
    self.name = key_name
    self.key_version_specified = bool(_KMS_KEY_VERSION_REGEX.match(key_name))

  def encrypt(self, plaintext: bytes, associated_data: bytes) -> bytes:
    try:
      response = self.client.encrypt(
          request=kms_v1.types.service.EncryptRequest(
              name=self.name,
              plaintext=plaintext,
              additional_authenticated_data=associated_data,
          )
      )
      return response.ciphertext
    except core_exceptions.GoogleAPIError as e:
      raise tink.TinkError(e)

  def decrypt(self, ciphertext: bytes, associated_data: bytes) -> bytes:
    if self.key_version_specified:
      raise tink.TinkError(
          'A CryptoKeyVersion was specified. Decryption is only supported when '
          'a CryptoKey is specified.'
      )
    try:
      response = self.client.decrypt(
         request=kms_v1.types.service.DecryptRequest(
             name=self.name,
             ciphertext=ciphertext,
             additional_authenticated_data=associated_data
         )
      )
      return response.plaintext
    except core_exceptions.GoogleAPIError as e:
      raise tink.TinkError(e)


class GcpKmsClient(tink.KmsClient):
  """Basic GCP client for AEAD."""

  def __init__(
      self,
      key_uri: Optional[str],
      credentials_path: Optional[str] = None,
      *,
      credentials: Optional[auth_credentials.Credentials] = None,
  ) -> None:
    """Creates a new GcpKmsClient that is bound to the key specified in 'key_uri'.

    Uses the specified credentials when communicating with the KMS. If neither
    credentials_path nor credentials are specified, the client will attempt to
    ascertain credentials from the environment.

    The key_uri can either by a CryptoKey or a CryptoKeyVersion. If a CryptoKey
    is specified, both encryption and decryption operations are supported. If a
    CryptoKeyVersion is specified, only encryption operations are supported.

    Args:
      key_uri: The URI of the key the client should be bound to. If it is None
        or empty, then the client is not bound to any particular key.
      credentials_path: Path to the file with the access credentials.
      credentials: The authorization credentials to attach to requests. This
        argument is mutually exclusive with credentials_path.

    Raises:
      ValueError: If the path or filename of the credentials is invalid.
      TinkError: If the key uri is not valid.
    """

    if not key_uri:
      self._key_uri = None
    elif key_uri.startswith(GCP_KEYURI_PREFIX):
      self._key_uri = key_uri
    else:
      raise tink.TinkError('Invalid key_uri.')
    if credentials and credentials_path:
      raise tink.TinkError(
          'Only one of credentials and credentials_path can be set.'
      )
    if not credentials and credentials_path:
      credentials = service_account.Credentials.from_service_account_file(
          credentials_path
      )
    self._client = kms_v1.KeyManagementServiceClient(credentials=credentials)

  def __enter__(self):
    return self

  def __exit__(self, exc_type, exc_value, traceback):
    if hasattr(self._client, '__exit__'):
      self._client.__exit__(exc_type, exc_value, traceback)
    elif hasattr(self._client, 'transport'):
      self._client.transport.channel.close()

  def does_support(self, key_uri: str) -> bool:
    """Returns true iff this client supports KMS key specified in 'key_uri'.

    Args:
      key_uri: URI of the key to be checked.

    Returns:
      A boolean value which is true if the key is supported and false otherwise.
    """
    if not self._key_uri:
      return key_uri.startswith(GCP_KEYURI_PREFIX)
    return key_uri == self._key_uri

  def get_aead(self, key_uri: str) -> aead.Aead:
    """Returns an Aead-primitive backed by KMS key specified by 'key_uri'.

    Args:
      key_uri: URI of the key which should be used.

    Returns:
      An Aead object.
    """
    if self._key_uri and self._key_uri != key_uri:
      raise tink.TinkError(
          'This client is bound to %s and cannot use key %s'
          % (self._key_uri, key_uri)
      )
    if not key_uri.startswith(GCP_KEYURI_PREFIX):
      raise tink.TinkError('Invalid key_uri.')
    key_id = key_uri[len(GCP_KEYURI_PREFIX) :]
    return _GcpKmsAead(self._client, key_id)

  @classmethod
  def register_client(
      cls, key_uri: Optional[str], credentials_path: Optional[str]
  ) -> None:
    """Add a new KMS client to the global list of KMS clients.

    This function should only be called on startup and not on every operation.

    In many cases, it is not necessary to register the client. For example,
    you can create a KMS AEAD with
    kms_aead = gcpkms.GcpKmsClient(key_uri, credentials_path).get_aead(key_uri)
    and then use it to encrypt a keyset with KeysetHandle.write, or to create
    an envelope AEAD using aead.KmsEnvelopeAead.

    Args:
        key_uri: Optional key URI. If set, the registered client will only
          handle that key URI. If not set, then the client will handle all AWS
          KMS key URIs.
        credentials_path: Optional path to the credentials file. If it is not
          set, the default credentials are used.
    """
    tink.register_kms_client(GcpKmsClient(key_uri, credentials_path))



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for secret_key_access."""

from absl.testing import absltest

import tink
from tink import aead
from tink import daead
from tink import hybrid
from tink import jwt
from tink import mac
from tink import prf
from tink import signature
from tink import streaming_aead


class SecretKeyAccessImportTest(absltest.TestCase):

  def test_tink_secret_key_access_not_imported(self):
    _ = tink
    _ = aead
    _ = daead
    _ = hybrid
    _ = jwt
    _ = mac
    _ = prf
    _ = signature
    _ = streaming_aead
    self.assertFalse(hasattr(tink, "secret_key_access"))


if __name__ == "__main__":
  absltest.main()



# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.integration.gcp_kms_aead."""

from absl.testing import absltest
from absl.testing import parameterized
from google.api_core import exceptions as core_exceptions
from google.cloud import kms_v1

from tink import core
from tink.integration.gcpkms import _gcp_kms_client


GCP_KEY_NAME = 'projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck1'
PLAINTEXT = b'plaintext'
CIPHERTEXT = b'ciphertext'
ASSOCIATED_DATA = b'associated_data'
STRING_63 = 'A' * 63
STRING_64 = 'A' * 64
FAKE_PROJECT_ID = '~@#$%^&*()_+|}{POI}?><213:"L{O}µ÷åß∑' * 5


class CustomException(core_exceptions.GoogleAPIError):
  pass


class GcpKmsAeadTest(parameterized.TestCase):

  def setUp(self):
    super().setUp()
    absltest.mock.patch.object(kms_v1, 'KeyManagementServiceClient').start()

  def tearDown(self):
    absltest.mock.patch.stopall()
    super().tearDown()

  def test_client_null(self):
    with self.assertRaises(core.TinkError):
      _gcp_kms_client._GcpKmsAead(None, GCP_KEY_NAME)

  @parameterized.parameters(
      '',
      None,
      'wrong/kms/key/format',
      'projects/p1/locations/global/keyRings/kr1/cryptoKeys',
      'projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck1/',
      'projects/p1/locations/global@/keyRings/kr1/cryptoKeys/ck1',
      'projects/p1/locations/global/keyRings/kr1@/cryptoKeys/ck1',
      'projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck1@',
      'projects/p1/locations/' + STRING_64 + '/keyRings/kr1/cryptoKeys/ck1',
      'projects/p1/locations/global/keyRings/' + STRING_64 + '/cryptoKeys/ck1',
      'projects/p1/locations/global/keyRings/kr1/cryptoKeys/' + STRING_64,
      'gcprojects://projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck1',
  )
  def test_key_name_format_wrong(self, key_name):
    with self.assertRaises(core.TinkError):
      _gcp_kms_client._GcpKmsAead(kms_v1.KeyManagementServiceClient(), key_name)

  def test_encryption_fails(self):
    kms_v1.KeyManagementServiceClient().encrypt.side_effect = CustomException()
    gcp_aead = _gcp_kms_client._GcpKmsAead(
        kms_v1.KeyManagementServiceClient(), GCP_KEY_NAME
    )
    with self.assertRaises(core.TinkError):
      gcp_aead.encrypt(CIPHERTEXT, ASSOCIATED_DATA)

  def test_decryption_fails(self):
    kms_v1.KeyManagementServiceClient().decrypt.side_effect = CustomException()
    gcp_aead = _gcp_kms_client._GcpKmsAead(
        kms_v1.KeyManagementServiceClient(), GCP_KEY_NAME
    )
    with self.assertRaises(core.TinkError):
      gcp_aead.decrypt(PLAINTEXT, ASSOCIATED_DATA)

  @parameterized.parameters(
      GCP_KEY_NAME,
      'projects/p1@comp!/locations/global/keyRings/kr1/cryptoKeys/ck1',
      'projects/' + FAKE_PROJECT_ID + '/locations/l1/keyRings/k1/cryptoKeys/c1',
      'projects/p1/locations/' + STRING_63 + '/keyRings/kr1/cryptoKeys/ck1',
      'projects/p1/locations/global/keyRings/' + STRING_63 + '/cryptoKeys/ck1',
      'projects/p1/locations/global/keyRings/kr1/cryptoKeys/' + STRING_63,
  )
  def test_encryption_works(self, key_name):
    kms_v1.KeyManagementServiceClient().encrypt.return_value = (
        kms_v1.types.EncryptResponse(name=GCP_KEY_NAME, ciphertext=CIPHERTEXT)
    )
    gcp_aead = _gcp_kms_client._GcpKmsAead(
        kms_v1.KeyManagementServiceClient(), key_name
    )
    ciphertext = gcp_aead.encrypt(PLAINTEXT, ASSOCIATED_DATA)
    self.assertEqual(ciphertext, CIPHERTEXT)

  def test_decryption_works(self):
    kms_v1.KeyManagementServiceClient().decrypt.return_value = (
        kms_v1.types.DecryptResponse(plaintext=PLAINTEXT)
    )
    gcp_aead = _gcp_kms_client._GcpKmsAead(
        kms_v1.KeyManagementServiceClient(), GCP_KEY_NAME
    )
    plaintext = gcp_aead.decrypt(CIPHERTEXT, ASSOCIATED_DATA)
    self.assertEqual(plaintext, PLAINTEXT)


if __name__ == '__main__':
  absltest.main()
